@@ -42,6 +42,7 @@ use url::Url;
 use distribution_filename::WheelFilename;
 use pep440_rs::Version;
 use pep508_rs::{Pep508Url, VerbatimUrl};
+use pypi_types::{ParsedUrl, VerbatimParsedUrl};
 use uv_git::GitUrl;
 use uv_normalize::PackageName;
 
@@ -49,14 +50,13 @@ pub use crate::annotation::*;
 pub use crate::any::*;
 pub use crate::buildable::*;
 pub use crate::cached::*;
-pub use crate::editable::*;
+pub use crate::diagnostic::*;
 pub use crate::error::*;
 pub use crate::file::*;
 pub use crate::hash::*;
 pub use crate::id::*;
 pub use crate::index_url::*;
 pub use crate::installed::*;
-pub use crate::parsed_url::*;
 pub use crate::prioritized_distribution::*;
 pub use crate::requirement::*;
 pub use crate::resolution::*;
@@ -68,14 +68,13 @@ mod annotation;
 mod any;
 mod buildable;
 mod cached;
-mod editable;
+mod diagnostic;
 mod error;
 mod file;
 mod hash;
 mod id;
 mod index_url;
 mod installed;
-mod parsed_url;
 mod prioritized_distribution;
 mod requirement;
 mod resolution;
@@ -286,9 +285,9 @@ impl Dist {
     /// URL.
     pub fn from_http_url(
         name: PackageName,
+        url: VerbatimUrl,
         location: Url,
         subdirectory: Option<PathBuf>,
-        url: VerbatimUrl,
     ) -> Result<Dist, Error> {
         if Path::new(url.path())
             .extension()
@@ -322,9 +321,9 @@ impl Dist {
     /// A local built or source distribution from a `file://` URL.
     pub fn from_file_url(
         name: PackageName,
+        url: VerbatimUrl,
         path: &Path,
         editable: bool,
-        url: VerbatimUrl,
     ) -> Result<Dist, Error> {
         // Store the canonicalized path, which also serves to validate that it exists.
         let path = match path.canonicalize() {
@@ -398,26 +397,17 @@ impl Dist {
     pub fn from_url(name: PackageName, url: VerbatimParsedUrl) -> Result<Self, Error> {
         match url.parsed_url {
             ParsedUrl::Archive(archive) => {
-                Self::from_http_url(name, archive.url, archive.subdirectory, url.verbatim)
+                Self::from_http_url(name, url.verbatim, archive.url, archive.subdirectory)
+            }
+            ParsedUrl::Path(file) => {
+                Self::from_file_url(name, url.verbatim, &file.path, file.editable)
             }
-            ParsedUrl::Path(file) => Self::from_file_url(name, &file.path, false, url.verbatim),
             ParsedUrl::Git(git) => {
                 Self::from_git_url(name, url.verbatim, git.url, git.subdirectory)
             }
         }
     }
 
-    /// Create a [`Dist`] for a local editable distribution.
-    pub fn from_editable(name: PackageName, editable: LocalEditable) -> Result<Self, Error> {
-        let LocalEditable { url, path, .. } = editable;
-        Ok(Self::Source(SourceDist::Directory(DirectorySourceDist {
-            name,
-            url,
-            path,
-            editable: true,
-        })))
-    }
-
     /// Return true if the distribution is editable.
     pub fn is_editable(&self) -> bool {
         match self {
@@ -426,6 +416,14 @@ impl Dist {
         }
     }
 
+    /// Return true if the distribution refers to a local file or directory.
+    pub fn is_local(&self) -> bool {
+        match self {
+            Self::Source(dist) => dist.is_local(),
+            Self::Built(dist) => dist.is_local(),
+        }
+    }
+
     /// Returns the [`IndexUrl`], if the distribution is from a registry.
     pub fn index(&self) -> Option<&IndexUrl> {
         match self {
@@ -451,6 +449,11 @@ impl Dist {
 }
 
 impl BuiltDist {
+    /// Return true if the distribution refers to a local file or directory.
+    pub fn is_local(&self) -> bool {
+        matches!(self, Self::Path(_))
+    }
+
     /// Returns the [`IndexUrl`], if the distribution is from a registry.
     pub fn index(&self) -> Option<&IndexUrl> {
         match self {
@@ -509,6 +512,11 @@ impl SourceDist {
         }
     }
 
+    /// Return true if the distribution refers to a local file or directory.
+    pub fn is_local(&self) -> bool {
+        matches!(self, Self::Directory(_) | Self::Path(_))
+    }
+
     /// Returns the path to the source distribution, if it's a local distribution.
     pub fn as_path(&self) -> Option<&Path> {
         match self {
