@@ -1550,6 +1550,11 @@ impl FromStr for MarkerTree {
 }
 
 impl MarkerTree {
+    /// Like [`FromStr::from_str`], but the caller chooses the return type generic.
+    pub fn parse_str<T: Pep508Url>(markers: &str) -> Result<Self, Pep508Error<T>> {
+        parse_markers(markers, &mut TracingReporter)
+    }
+
     /// Parse a [`MarkerTree`] from a string with the given reporter.
     pub fn parse_reporter(
         markers: &str,
@@ -1597,7 +1602,7 @@ impl MarkerTree {
     /// the marker will be simplified to `sys_platform == 'linux'`.
     pub fn simplify_extras(self, extras: &[ExtraName]) -> Option<MarkerTree> {
         /// Returns `true` if the given expression is always `true` given the set of extras.
-        pub fn is_true(expression: &MarkerExpression, extras: &[ExtraName]) -> bool {
+        fn is_true(expression: &MarkerExpression, extras: &[ExtraName]) -> bool {
             match expression {
                 MarkerExpression::Extra {
                     operator: ExtraOperator::Equal,
