@@ -14,7 +14,7 @@ use crate::Pep508Url;
 
 /// A wrapper around [`Url`] that preserves the original string.
 #[derive(Debug, Clone, Eq, derivative::Derivative, serde::Deserialize, serde::Serialize)]
-#[derivative(PartialEq, Hash)]
+#[derivative(PartialEq, Hash, Ord)]
 pub struct VerbatimUrl {
     /// The parsed URL.
     #[serde(
@@ -24,6 +24,7 @@ pub struct VerbatimUrl {
     url: Url,
     /// The URL as it was provided by the user.
     #[derivative(PartialEq = "ignore")]
+    #[derivative(Ord = "ignore")]
     #[derivative(Hash = "ignore")]
     given: Option<String>,
 }
@@ -37,25 +38,26 @@ impl VerbatimUrl {
     /// Create a [`VerbatimUrl`] from a file path.
     ///
     /// Assumes that the path is absolute.
-    pub fn from_path(path: impl AsRef<Path>) -> Self {
+    pub fn from_path(path: impl AsRef<Path>) -> Result<Self, VerbatimUrlError> {
         let path = path.as_ref();
 
         // Normalize the path.
-        let path = normalize_path(path).expect("path is absolute");
+        let path = normalize_path(path)
+            .map_err(|err| VerbatimUrlError::Normalization(path.to_path_buf(), err))?;
 
         // Extract the fragment, if it exists.
         let (path, fragment) = split_fragment(&path);
 
         // Convert to a URL.
         let mut url = Url::from_file_path(path.clone())
-            .unwrap_or_else(|_| panic!("path is absolute: {}", path.display()));
+            .map_err(|_| VerbatimUrlError::UrlConversion(path.to_path_buf()))?;
 
         // Set the fragment, if it exists.
         if let Some(fragment) = fragment {
             url.set_fragment(Some(fragment));
         }
 
-        Self { url, given: None }
+        Ok(Self { url, given: None })
     }
 
     /// Parse a URL from a string, expanding any environment variables.
@@ -66,37 +68,37 @@ impl VerbatimUrl {
 
     /// Parse a URL from an absolute or relative path.
     #[cfg(feature = "non-pep508-extensions")] // PEP 508 arguably only allows absolute file URLs.
-    pub fn parse_path(path: impl AsRef<Path>, working_dir: impl AsRef<Path>) -> Self {
+    pub fn parse_path(
+        path: impl AsRef<Path>,
+        base_dir: impl AsRef<Path>,
+    ) -> Result<Self, VerbatimUrlError> {
+        debug_assert!(base_dir.as_ref().is_absolute(), "base dir must be absolute");
         let path = path.as_ref();
 
         // Convert the path to an absolute path, if necessary.
         let path = if path.is_absolute() {
             path.to_path_buf()
         } else {
-            working_dir.as_ref().join(path)
+            base_dir.as_ref().join(path)
         };
 
         // Normalize the path.
-        let path = normalize_path(&path).expect("path is absolute");
+        let path = normalize_path(&path)
+            .map_err(|err| VerbatimUrlError::Normalization(path.to_path_buf(), err))?;
 
         // Extract the fragment, if it exists.
         let (path, fragment) = split_fragment(&path);
 
         // Convert to a URL.
-        let mut url = Url::from_file_path(path.clone()).unwrap_or_else(|_| {
-            panic!(
-                "path is absolute: {}, {}",
-                path.display(),
-                working_dir.as_ref().display()
-            )
-        });
+        let mut url = Url::from_file_path(path.clone())
+            .map_err(|_| VerbatimUrlError::UrlConversion(path.to_path_buf()))?;
 
         // Set the fragment, if it exists.
         if let Some(fragment) = fragment {
             url.set_fragment(Some(fragment));
         }
 
-        Self { url, given: None }
+        Ok(Self { url, given: None })
     }
 
     /// Parse a URL from an absolute path.
@@ -107,12 +109,12 @@ impl VerbatimUrl {
         let path = if path.is_absolute() {
             path.to_path_buf()
         } else {
-            return Err(VerbatimUrlError::RelativePath(path.to_path_buf()));
+            return Err(VerbatimUrlError::WorkingDirectory(path.to_path_buf()));
         };
 
         // Normalize the path.
         let Ok(path) = normalize_path(&path) else {
-            return Err(VerbatimUrlError::RelativePath(path));
+            return Err(VerbatimUrlError::WorkingDirectory(path));
         };
 
         // Extract the fragment, if it exists.
@@ -153,13 +155,15 @@ impl VerbatimUrl {
     pub fn to_url(&self) -> Url {
         self.url.clone()
     }
+}
 
-    /// Create a [`VerbatimUrl`] from a [`Url`].
-    ///
-    /// This method should be used sparingly (ideally, not at all), as it represents a loss of the
-    /// verbatim representation.
-    pub fn unknown(url: Url) -> Self {
-        Self { given: None, url }
+// This impl is written out because the `derive` doesn't seem to get it right.
+// Or does it in a way where Clippy complains about non-canonical `PartialOrd`
+// impls. So we just do it by hand by deferring to the derived `Ord` impl. This
+// guarantees they are consistent.
+impl PartialOrd for VerbatimUrl {
+    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
+        Some(self.cmp(other))
     }
 }
 
@@ -215,7 +219,7 @@ impl Pep508Url for VerbatimUrl {
 
                     #[cfg(feature = "non-pep508-extensions")]
                     if let Some(working_dir) = working_dir {
-                        return Ok(VerbatimUrl::parse_path(path.as_ref(), working_dir)
+                        return Ok(VerbatimUrl::parse_path(path.as_ref(), working_dir)?
                             .with_given(url.to_string()));
                     }
 
@@ -234,7 +238,7 @@ impl Pep508Url for VerbatimUrl {
                 _ => {
                     #[cfg(feature = "non-pep508-extensions")]
                     if let Some(working_dir) = working_dir {
-                        return Ok(VerbatimUrl::parse_path(expanded.as_ref(), working_dir)
+                        return Ok(VerbatimUrl::parse_path(expanded.as_ref(), working_dir)?
                             .with_given(url.to_string()));
                     }
 
@@ -246,7 +250,7 @@ impl Pep508Url for VerbatimUrl {
             // Ex) `../editable/`
             #[cfg(feature = "non-pep508-extensions")]
             if let Some(working_dir) = working_dir {
-                return Ok(VerbatimUrl::parse_path(expanded.as_ref(), working_dir)
+                return Ok(VerbatimUrl::parse_path(expanded.as_ref(), working_dir)?
                     .with_given(url.to_string()));
             }
 
@@ -264,7 +268,15 @@ pub enum VerbatimUrlError {
 
     /// Received a relative path, but no working directory was provided.
     #[error("relative path without a working directory: {0}")]
-    RelativePath(PathBuf),
+    WorkingDirectory(PathBuf),
+
+    /// Received a path that could not be converted to a URL.
+    #[error("path could not be converted to a URL: {0}")]
+    UrlConversion(PathBuf),
+
+    /// Received a path that could not be normalized.
+    #[error("path could not be normalized: {0}")]
+    Normalization(PathBuf, #[source] std::io::Error),
 }
 
 /// Expand all available environment variables.
