@@ -44,13 +44,12 @@ use tracing::instrument;
 use unscanny::{Pattern, Scanner};
 use url::Url;
 
-use distribution_types::{
-    ParsedUrlError, Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification,
-};
+use distribution_types::{Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification};
 use pep508_rs::{
     expand_env_vars, split_scheme, strip_host, Extras, MarkerTree, Pep508Error, Pep508ErrorSource,
-    RequirementOrigin, Scheme, VerbatimUrl,
+    RequirementOrigin, Scheme, UnnamedRequirement, VerbatimUrl,
 };
+use pypi_types::{ParsedPathUrl, ParsedUrl, VerbatimParsedUrl};
 #[cfg(feature = "http")]
 use uv_client::BaseClient;
 use uv_client::BaseClientBuilder;
@@ -59,7 +58,7 @@ use uv_fs::{normalize_url_path, Simplified};
 use uv_normalize::ExtraName;
 use uv_warnings::warn_user;
 
-pub use crate::requirement::{RequirementsTxtRequirement, RequirementsTxtRequirementError};
+pub use crate::requirement::RequirementsTxtRequirement;
 
 mod requirement;
 
@@ -203,7 +202,7 @@ impl EditableRequirement {
     ) -> Result<Self, RequirementsTxtParserError> {
         // Identify the markers.
         let (given, marker) = if let Some((requirement, marker)) = Self::split_markers(given) {
-            let marker = MarkerTree::from_str(marker).map_err(|err| {
+            let marker = MarkerTree::parse_str(marker).map_err(|err| {
                 // Map from error on the markers to error on the whole requirement.
                 let err = Pep508Error {
                     message: err.message,
@@ -216,14 +215,14 @@ impl EditableRequirement {
                         RequirementsTxtParserError::Pep508 {
                             start: err.start,
                             end: err.start + err.len,
-                            source: err,
+                            source: Box::new(err),
                         }
                     }
                     Pep508ErrorSource::UnsupportedRequirement(_) => {
                         RequirementsTxtParserError::UnsupportedRequirement {
                             start: err.start,
                             end: err.start + err.len,
-                            source: err,
+                            source: Box::new(err),
                         }
                     }
                 }
@@ -248,14 +247,14 @@ impl EditableRequirement {
                         RequirementsTxtParserError::Pep508 {
                             start: err.start,
                             end: err.start + err.len,
-                            source: err,
+                            source: Box::new(err),
                         }
                     }
                     Pep508ErrorSource::UnsupportedRequirement(_) => {
                         RequirementsTxtParserError::UnsupportedRequirement {
                             start: err.start,
                             end: err.start + err.len,
-                            source: err,
+                            source: Box::new(err),
                         }
                     }
                 }
@@ -403,21 +402,40 @@ pub struct RequirementEntry {
 // We place the impl here instead of next to `UnresolvedRequirementSpecification` because
 // `UnresolvedRequirementSpecification` is defined in `distribution-types` and `requirements-txt`
 // depends on `distribution-types`.
-impl TryFrom<RequirementEntry> for UnresolvedRequirementSpecification {
-    type Error = Box<ParsedUrlError>;
-
-    fn try_from(value: RequirementEntry) -> Result<Self, Self::Error> {
-        Ok(Self {
+impl From<RequirementEntry> for UnresolvedRequirementSpecification {
+    fn from(value: RequirementEntry) -> Self {
+        Self {
             requirement: match value.requirement {
                 RequirementsTxtRequirement::Named(named) => {
-                    UnresolvedRequirement::Named(Requirement::from_pep508(named)?)
+                    UnresolvedRequirement::Named(Requirement::from(named))
                 }
                 RequirementsTxtRequirement::Unnamed(unnamed) => {
                     UnresolvedRequirement::Unnamed(unnamed)
                 }
             },
             hashes: value.hashes,
-        })
+        }
+    }
+}
+
+impl From<EditableRequirement> for UnresolvedRequirementSpecification {
+    fn from(value: EditableRequirement) -> Self {
+        Self {
+            requirement: UnresolvedRequirement::Unnamed(UnnamedRequirement {
+                url: VerbatimParsedUrl {
+                    parsed_url: ParsedUrl::Path(ParsedPathUrl {
+                        url: value.url.to_url(),
+                        path: value.path,
+                        editable: true,
+                    }),
+                    verbatim: value.url,
+                },
+                extras: value.extras,
+                marker: value.marker,
+                origin: value.origin,
+            }),
+            hashes: vec![],
+        }
     }
 }
 
@@ -427,7 +445,7 @@ pub struct RequirementsTxt {
     /// The actual requirements with the hashes.
     pub requirements: Vec<RequirementEntry>,
     /// Constraints included with `-c`.
-    pub constraints: Vec<pep508_rs::Requirement>,
+    pub constraints: Vec<pep508_rs::Requirement<VerbatimParsedUrl>>,
     /// Editables with `-e`.
     pub editables: Vec<EditableRequirement>,
     /// The index URL, specified with `--index-url`.
@@ -914,30 +932,10 @@ fn parse_requirement_and_hashes(
                 requirement
             }
         })
-        .map_err(|err| match err {
-            RequirementsTxtRequirementError::ParsedUrl(err) => {
-                RequirementsTxtParserError::ParsedUrl {
-                    source: err,
-                    start,
-                    end,
-                }
-            }
-            RequirementsTxtRequirementError::Pep508(err) => match err.message {
-                Pep508ErrorSource::String(_) | Pep508ErrorSource::UrlError(_) => {
-                    RequirementsTxtParserError::Pep508 {
-                        source: err,
-                        start,
-                        end,
-                    }
-                }
-                Pep508ErrorSource::UnsupportedRequirement(_) => {
-                    RequirementsTxtParserError::UnsupportedRequirement {
-                        source: err,
-                        start,
-                        end,
-                    }
-                }
-            },
+        .map_err(|err| RequirementsTxtParserError::Pep508 {
+            source: err,
+            start,
+            end,
         })?;
 
     let hashes = if has_hashes {
@@ -1068,17 +1066,17 @@ pub enum RequirementsTxtParserError {
         column: usize,
     },
     UnsupportedRequirement {
-        source: Pep508Error,
+        source: Box<Pep508Error<VerbatimParsedUrl>>,
         start: usize,
         end: usize,
     },
     Pep508 {
-        source: Pep508Error,
+        source: Box<Pep508Error<VerbatimParsedUrl>>,
         start: usize,
         end: usize,
     },
     ParsedUrl {
-        source: Box<ParsedUrlError>,
+        source: Box<Pep508Error<VerbatimParsedUrl>>,
         start: usize,
         end: usize,
     },
