@@ -25,9 +25,10 @@ use tokio::process::Command;
 use tokio::sync::{Mutex, Semaphore};
 use tracing::{debug, info_span, instrument, Instrument};
 
-use distribution_types::{ParsedUrlError, Requirement, Resolution};
+use distribution_types::{Requirement, Resolution};
 use pep440_rs::Version;
 use pep508_rs::PackageName;
+use pypi_types::VerbatimParsedUrl;
 use uv_configuration::{BuildKind, ConfigSettings, SetupPyStrategy};
 use uv_fs::{PythonExt, Simplified};
 use uv_interpreter::{Interpreter, PythonEnvironment};
@@ -66,18 +67,16 @@ static WHEEL_NOT_FOUND_RE: Lazy<Regex> =
 static DEFAULT_BACKEND: Lazy<Pep517Backend> = Lazy::new(|| Pep517Backend {
     backend: "setuptools.build_meta:__legacy__".to_string(),
     backend_path: None,
-    requirements: vec![Requirement::from_pep508(
+    requirements: vec![Requirement::from(
         pep508_rs::Requirement::from_str("setuptools >= 40.8.0").unwrap(),
-    )
-    .unwrap()],
+    )],
 });
 
 /// The requirements for `--legacy-setup-py` builds.
 static SETUP_PY_REQUIREMENTS: Lazy<[Requirement; 2]> = Lazy::new(|| {
     [
-        Requirement::from_pep508(pep508_rs::Requirement::from_str("setuptools >= 40.8.0").unwrap())
-            .unwrap(),
-        Requirement::from_pep508(pep508_rs::Requirement::from_str("wheel").unwrap()).unwrap(),
+        Requirement::from(pep508_rs::Requirement::from_str("setuptools >= 40.8.0").unwrap()),
+        Requirement::from(pep508_rs::Requirement::from_str("wheel").unwrap()),
     ]
 });
 
@@ -116,8 +115,6 @@ pub enum Error {
     },
     #[error("Failed to build PATH for build script")]
     BuildScriptPath(#[source] env::JoinPathsError),
-    #[error("Failed to parse requirements from build backend")]
-    DirectUrl(#[source] Box<ParsedUrlError>),
 }
 
 #[derive(Debug)]
@@ -244,7 +241,7 @@ pub struct Project {
 #[serde(rename_all = "kebab-case")]
 pub struct BuildSystem {
     /// PEP 508 dependencies required to execute the build system.
-    pub requires: Vec<pep508_rs::Requirement>,
+    pub requires: Vec<pep508_rs::Requirement<VerbatimParsedUrl>>,
     /// A string naming a Python object that will be used to perform the build.
     pub build_backend: Option<String>,
     /// Specify that their backend code is hosted in-tree, this key contains a list of directories.
@@ -601,9 +598,8 @@ impl SourceBuild {
                         requirements: build_system
                             .requires
                             .into_iter()
-                            .map(Requirement::from_pep508)
-                            .collect::<Result<_, _>>()
-                            .map_err(|err| Box::new(Error::DirectUrl(err)))?,
+                            .map(Requirement::from)
+                            .collect(),
                     }
                 } else {
                     // If a `pyproject.toml` is present, but `[build-system]` is missing, proceed with
@@ -681,23 +677,23 @@ impl SourceBuild {
         fs::create_dir(&metadata_directory)?;
 
         // Write the hook output to a file so that we can read it back reliably.
-        let outfile = self
-            .temp_dir
-            .path()
-            .join("prepare_metadata_for_build_wheel.txt");
+        let outfile = self.temp_dir.path().join(format!(
+            "prepare_metadata_for_build_{}.txt",
+            self.build_kind
+        ));
 
         debug!(
-            "Calling `{}.prepare_metadata_for_build_wheel()`",
-            pep517_backend.backend
+            "Calling `{}.prepare_metadata_for_build_{}()`",
+            pep517_backend.backend, self.build_kind,
         );
         let script = formatdoc! {
             r#"
             {}
             import json
 
-            prepare_metadata_for_build_wheel = getattr(backend, "prepare_metadata_for_build_wheel", None)
-            if prepare_metadata_for_build_wheel:
-                dirname = prepare_metadata_for_build_wheel("{}", {})
+            prepare_metadata_for_build = getattr(backend, "prepare_metadata_for_build_{}", None)
+            if prepare_metadata_for_build:
+                dirname = prepare_metadata_for_build("{}", {})
             else:
                 dirname = None
 
@@ -705,13 +701,14 @@ impl SourceBuild {
                 fp.write(dirname or "")
             "#,
             pep517_backend.backend_import(),
+            self.build_kind,
             escape_path_for_python(&metadata_directory),
             self.config_settings.escape_for_python(),
             outfile.escape_for_python(),
         };
         let span = info_span!(
             "run_python_script",
-            script="prepare_metadata_for_build_wheel",
+            script=format!("prepare_metadata_for_build_{}", self.build_kind),
             python_version = %self.venv.interpreter().python_version()
         );
         let output = self
@@ -727,7 +724,7 @@ impl SourceBuild {
             .await?;
         if !output.status.success() {
             return Err(Error::from_command_output(
-                "Build backend failed to determine metadata through `prepare_metadata_for_build_wheel`".to_string(),
+                format!("Build backend failed to determine metadata through `prepare_metadata_for_build_{}`", self.build_kind),
                 &output,
                 &self.version_id,
             ));
@@ -982,7 +979,7 @@ async fn create_pep517_build_environment(
     })?;
 
     // Deserialize the requirements from the output file.
-    let extra_requires: Vec<pep508_rs::Requirement> = serde_json::from_slice::<Vec<pep508_rs::Requirement>>(&contents).map_err(|err| {
+    let extra_requires: Vec<pep508_rs::Requirement<VerbatimParsedUrl>> = serde_json::from_slice::<Vec<pep508_rs::Requirement<VerbatimParsedUrl>>>(&contents).map_err(|err| {
         Error::from_command_output(
             format!(
                 "Build backend failed to return extra requires with `get_requires_for_build_{build_kind}`: {err}"
@@ -991,11 +988,7 @@ async fn create_pep517_build_environment(
             version_id,
         )
     })?;
-    let extra_requires: Vec<_> = extra_requires
-        .into_iter()
-        .map(Requirement::from_pep508)
-        .collect::<Result<_, _>>()
-        .map_err(Error::DirectUrl)?;
+    let extra_requires: Vec<_> = extra_requires.into_iter().map(Requirement::from).collect();
 
     // Some packages (such as tqdm 4.66.1) list only extra requires that have already been part of
     // the pyproject.toml requires (in this case, `wheel`). We can skip doing the whole resolution
