@@ -2,7 +2,6 @@
 //! [installer][`uv_installer`] and [build][`uv_build`] through [`BuildDispatch`]
 //! implementing [`BuildContext`].
 
-use std::ffi::OsStr;
 use std::ffi::OsString;
 use std::path::Path;
 
@@ -12,7 +11,7 @@ use itertools::Itertools;
 use rustc_hash::FxHashMap;
 use tracing::{debug, instrument};
 
-use distribution_types::{IndexLocations, Name, Requirement, Resolution, SourceDist};
+use distribution_types::{CachedDist, IndexLocations, Name, Requirement, Resolution, SourceDist};
 use uv_build::{SourceBuild, SourceBuildContext};
 use uv_cache::Cache;
 use uv_client::RegistryClient;
@@ -90,21 +89,6 @@ impl<'a> BuildDispatch<'a> {
         self.options = options;
         self
     }
-
-    /// Set the environment variables to be used when building a source distribution.
-    #[must_use]
-    pub fn with_build_extra_env_vars<I, K, V>(mut self, sdist_build_env_variables: I) -> Self
-    where
-        I: IntoIterator<Item = (K, V)>,
-        K: AsRef<OsStr>,
-        V: AsRef<OsStr>,
-    {
-        self.build_extra_env_vars = sdist_build_env_variables
-            .into_iter()
-            .map(|(key, value)| (key.as_ref().to_owned(), value.as_ref().to_owned()))
-            .collect();
-        self
-    }
 }
 
 impl<'a> BuildContext for BuildDispatch<'a> {
@@ -153,7 +137,7 @@ impl<'a> BuildContext for BuildDispatch<'a> {
             self.index,
             &HashStrategy::None,
             self,
-            &EmptyInstalledPackages,
+            EmptyInstalledPackages,
             DistributionDatabase::new(self.client, self, self.concurrency.downloads),
         )?;
         let graph = resolver.resolve().await.with_context(|| {
@@ -176,7 +160,7 @@ impl<'a> BuildContext for BuildDispatch<'a> {
         &'data self,
         resolution: &'data Resolution,
         venv: &'data PythonEnvironment,
-    ) -> Result<()> {
+    ) -> Result<Vec<CachedDist>> {
         debug!(
             "Installing in {} in {}",
             resolution
@@ -192,12 +176,14 @@ impl<'a> BuildContext for BuildDispatch<'a> {
         // Determine the set of installed packages.
         let site_packages = SitePackages::from_executable(venv)?;
 
+        let requirements = resolution.requirements().collect::<Vec<_>>();
+
         let Plan {
             cached,
             remote,
             reinstalls,
             extraneous: _,
-        } = Planner::with_requirements(&resolution.requirements()).build(
+        } = Planner::new(&requirements).build(
             site_packages,
             &Reinstall::None,
             &NoBinary::None,
@@ -211,7 +197,7 @@ impl<'a> BuildContext for BuildDispatch<'a> {
         // Nothing to do.
         if remote.is_empty() && cached.is_empty() && reinstalls.is_empty() {
             debug!("No build requirements to install for build");
-            return Ok(());
+            return Ok(vec![]);
         }
 
         // Resolve any registry-based requirements.
@@ -280,7 +266,7 @@ impl<'a> BuildContext for BuildDispatch<'a> {
                 .context("Failed to install build dependencies")?;
         }
 
-        Ok(())
+        Ok(wheels)
     }
 
     #[instrument(skip_all, fields(version_id = version_id, subdirectory = ?subdirectory))]
