@@ -1,24 +1,26 @@
 use std::future::Future;
 use std::io;
 use std::path::Path;
+use std::pin::Pin;
 use std::rc::Rc;
 use std::sync::Arc;
+use std::task::{Context, Poll};
 
 use futures::{FutureExt, TryStreamExt};
 use tempfile::TempDir;
-use tokio::io::AsyncSeekExt;
+use tokio::io::{AsyncRead, AsyncSeekExt, ReadBuf};
 use tokio::sync::Semaphore;
 use tokio_util::compat::FuturesAsyncReadCompatExt;
-use tracing::{info_span, instrument, warn, Instrument};
+use tracing::{debug, info_span, instrument, warn, Instrument};
 use url::Url;
 
 use distribution_filename::WheelFilename;
 use distribution_types::{
-    BuildableSource, BuiltDist, Dist, FileLocation, HashPolicy, Hashed, IndexLocations,
-    LocalEditable, Name, SourceDist,
+    BuildableSource, BuiltDist, Dist, FileLocation, HashPolicy, Hashed, IndexLocations, Name,
+    SourceDist,
 };
 use platform_tags::Tags;
-use pypi_types::{HashDigest, Metadata23};
+use pypi_types::HashDigest;
 use uv_cache::{ArchiveId, ArchiveTimestamp, CacheBucket, CacheEntry, Timestamp, WheelCache};
 use uv_client::{
     CacheControl, CachedClientError, Connectivity, DataWithCachePolicy, RegistryClient,
@@ -49,6 +51,7 @@ pub struct DistributionDatabase<'a, Context: BuildContext> {
     builder: SourceDistributionBuilder<'a, Context>,
     locks: Rc<Locks>,
     client: ManagedClient<'a>,
+    reporter: Option<Arc<dyn Reporter>>,
 }
 
 impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
@@ -62,6 +65,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
             builder: SourceDistributionBuilder::new(build_context),
             locks: Rc::new(Locks::default()),
             client: ManagedClient::new(client, concurrent_downloads),
+            reporter: None,
         }
     }
 
@@ -70,6 +74,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
     pub fn with_reporter(self, reporter: impl Reporter + 'static) -> Self {
         let reporter = Arc::new(reporter);
         Self {
+            reporter: Some(reporter.clone()),
             builder: self.builder.with_reporter(reporter),
             ..self
         }
@@ -128,32 +133,6 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         }
     }
 
-    /// Build a directory into an editable wheel.
-    pub async fn build_wheel_editable(
-        &self,
-        editable: &LocalEditable,
-        editable_wheel_dir: &Path,
-    ) -> Result<(LocalWheel, Metadata23), Error> {
-        // Build the wheel.
-        let (dist, disk_filename, filename, metadata) = self
-            .builder
-            .build_editable(editable, editable_wheel_dir)
-            .await?;
-
-        // Unzip into the editable wheel directory.
-        let path = editable_wheel_dir.join(&disk_filename);
-        let target = editable_wheel_dir.join(cache_key::digest(&editable.path));
-        let id = self.unzip_wheel(&path, &target).await?;
-        let wheel = LocalWheel {
-            dist,
-            filename,
-            archive: self.build_context.cache().archive(&id),
-            hashes: vec![],
-        };
-
-        Ok((wheel, metadata))
-    }
-
     /// Fetch a wheel from the cache or download it from the index.
     ///
     /// While hashes will be generated in all cases, hash-checking is _not_ enforced and should
@@ -168,6 +147,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
             NoBinary::All => true,
             NoBinary::Packages(packages) => packages.contains(dist.name()),
         };
+
         if no_binary {
             return Err(Error::NoBinary);
         }
@@ -188,6 +168,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                             WheelCache::Index(&wheel.index).wheel_dir(wheel.name().as_ref()),
                             wheel.filename.stem(),
                         );
+
                         return self
                             .load_wheel(path, &wheel.filename, cache_entry, dist, hashes)
                             .await;
@@ -203,7 +184,14 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
 
                 // Download and unzip.
                 match self
-                    .stream_wheel(url.clone(), &wheel.filename, &wheel_entry, dist, hashes)
+                    .stream_wheel(
+                        url.clone(),
+                        &wheel.filename,
+                        wheel.file.size,
+                        &wheel_entry,
+                        dist,
+                        hashes,
+                    )
                     .await
                 {
                     Ok(archive) => Ok(LocalWheel {
@@ -220,8 +208,16 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                         // If the request failed because streaming is unsupported, download the
                         // wheel directly.
                         let archive = self
-                            .download_wheel(url, &wheel.filename, &wheel_entry, dist, hashes)
+                            .download_wheel(
+                                url,
+                                &wheel.filename,
+                                wheel.file.size,
+                                &wheel_entry,
+                                dist,
+                                hashes,
+                            )
                             .await?;
+
                         Ok(LocalWheel {
                             dist: Dist::Built(dist.clone()),
                             archive: self.build_context.cache().archive(&archive.id),
@@ -246,6 +242,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                     .stream_wheel(
                         wheel.url.raw().clone(),
                         &wheel.filename,
+                        None,
                         &wheel_entry,
                         dist,
                         hashes,
@@ -269,6 +266,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                             .download_wheel(
                                 wheel.url.raw().clone(),
                                 &wheel.filename,
+                                None,
                                 &wheel_entry,
                                 dist,
                                 hashes,
@@ -408,7 +406,11 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
 
         // Optimization: Skip source dist download when we must not build them anyway.
         if no_build {
-            return Err(Error::NoBuild);
+            if source.is_editable() {
+                debug!("Allowing build for editable source distribution: {source}");
+            } else {
+                return Err(Error::NoBuild);
+            }
         }
 
         let lock = self.locks.acquire(source).await;
@@ -419,6 +421,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
             .download_and_build_metadata(source, hashes, &self.client)
             .boxed_local()
             .await?;
+
         Ok(metadata)
     }
 
@@ -427,6 +430,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         &self,
         url: Url,
         filename: &WheelFilename,
+        size: Option<u64>,
         wheel_entry: &CacheEntry,
         dist: &BuiltDist,
         hashes: HashPolicy<'_>,
@@ -434,8 +438,19 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         // Create an entry for the HTTP cache.
         let http_entry = wheel_entry.with_file(format!("{}.http", filename.stem()));
 
+        // Fetch the archive from the cache, or download it if necessary.
+        let req = self.request(url.clone())?;
+
+        // Extract the size from the `Content-Length` header, if not provided by the registry.
+        let size = size.or_else(|| content_length(&req));
+
         let download = |response: reqwest::Response| {
             async {
+                let progress = self
+                    .reporter
+                    .as_ref()
+                    .map(|reporter| (reporter, reporter.on_download_start(dist.name(), size)));
+
                 let reader = response
                     .bytes_stream()
                     .map_err(|err| self.handle_response_errors(err))
@@ -449,7 +464,16 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                 // Download and unzip the wheel to a temporary directory.
                 let temp_dir = tempfile::tempdir_in(self.build_context.cache().root())
                     .map_err(Error::CacheWrite)?;
-                uv_extract::stream::unzip(&mut hasher, temp_dir.path()).await?;
+
+                match progress {
+                    Some((reporter, progress)) => {
+                        let mut reader = ProgressReader::new(&mut hasher, progress, &**reporter);
+                        uv_extract::stream::unzip(&mut reader, temp_dir.path()).await?;
+                    }
+                    None => {
+                        uv_extract::stream::unzip(&mut hasher, temp_dir.path()).await?;
+                    }
+                }
 
                 // If necessary, exhaust the reader to compute the hash.
                 if !hashes.is_none() {
@@ -464,6 +488,10 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                     .await
                     .map_err(Error::CacheRead)?;
 
+                if let Some((reporter, progress)) = progress {
+                    reporter.on_download_complete(dist.name(), progress);
+                }
+
                 Ok(Archive::new(
                     id,
                     hashers.into_iter().map(HashDigest::from).collect(),
@@ -523,6 +551,7 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         &self,
         url: Url,
         filename: &WheelFilename,
+        size: Option<u64>,
         wheel_entry: &CacheEntry,
         dist: &BuiltDist,
         hashes: HashPolicy<'_>,
@@ -530,8 +559,18 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
         // Create an entry for the HTTP cache.
         let http_entry = wheel_entry.with_file(format!("{}.http", filename.stem()));
 
+        let req = self.request(url.clone())?;
+
+        // Extract the size from the `Content-Length` header, if not provided by the registry.
+        let size = size.or_else(|| content_length(&req));
+
         let download = |response: reqwest::Response| {
             async {
+                let progress = self
+                    .reporter
+                    .as_ref()
+                    .map(|reporter| (reporter, reporter.on_download_start(dist.name(), size)));
+
                 let reader = response
                     .bytes_stream()
                     .map_err(|err| self.handle_response_errors(err))
@@ -541,9 +580,25 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                 let temp_file = tempfile::tempfile_in(self.build_context.cache().root())
                     .map_err(Error::CacheWrite)?;
                 let mut writer = tokio::io::BufWriter::new(tokio::fs::File::from_std(temp_file));
-                tokio::io::copy(&mut reader.compat(), &mut writer)
-                    .await
-                    .map_err(Error::CacheWrite)?;
+
+                match progress {
+                    Some((reporter, progress)) => {
+                        // Wrap the reader in a progress reporter. This will report 100% progress
+                        // after the download is complete, even if we still have to unzip and hash
+                        // part of the file.
+                        let mut reader =
+                            ProgressReader::new(reader.compat(), progress, &**reporter);
+
+                        tokio::io::copy(&mut reader, &mut writer)
+                            .await
+                            .map_err(Error::CacheWrite)?;
+                    }
+                    None => {
+                        tokio::io::copy(&mut reader.compat(), &mut writer)
+                            .await
+                            .map_err(Error::CacheWrite)?;
+                    }
+                }
 
                 // Unzip the wheel to a temporary directory.
                 let temp_dir = tempfile::tempdir_in(self.build_context.cache().root())
@@ -588,6 +643,10 @@ impl<'a, Context: BuildContext> DistributionDatabase<'a, Context> {
                     .await
                     .map_err(Error::CacheRead)?;
 
+                if let Some((reporter, progress)) = progress {
+                    reporter.on_download_complete(dist.name(), progress);
+                }
+
                 Ok(Archive::new(id, hashes))
             }
             .instrument(info_span!("wheel", wheel = %dist))
@@ -813,6 +872,50 @@ impl<'a> ManagedClient<'a> {
     }
 }
 
+/// Returns the value of the `Content-Length` header from the [`reqwest::Request`], if present.
+fn content_length(req: &reqwest::Request) -> Option<u64> {
+    req.headers()
+        .get(reqwest::header::CONTENT_LENGTH)
+        .and_then(|val| val.to_str().ok())
+        .and_then(|val| val.parse::<u64>().ok())
+}
+
+/// An asynchronous reader that reports progress as bytes are read.
+struct ProgressReader<'a, R> {
+    reader: R,
+    index: usize,
+    reporter: &'a dyn Reporter,
+}
+
+impl<'a, R> ProgressReader<'a, R> {
+    /// Create a new [`ProgressReader`] that wraps another reader.
+    fn new(reader: R, index: usize, reporter: &'a dyn Reporter) -> Self {
+        Self {
+            reader,
+            index,
+            reporter,
+        }
+    }
+}
+
+impl<R> AsyncRead for ProgressReader<'_, R>
+where
+    R: AsyncRead + Unpin,
+{
+    fn poll_read(
+        mut self: Pin<&mut Self>,
+        cx: &mut Context<'_>,
+        buf: &mut ReadBuf<'_>,
+    ) -> Poll<io::Result<()>> {
+        Pin::new(&mut self.as_mut().reader)
+            .poll_read(cx, buf)
+            .map_ok(|()| {
+                self.reporter
+                    .on_download_progress(self.index, buf.filled().len() as u64);
+            })
+    }
+}
+
 /// A pointer to an archive in the cache, fetched from an HTTP archive.
 ///
 /// Encoded with `MsgPack`, and represented on disk by a `.http` file.
