@@ -2,13 +2,14 @@ use itertools::Itertools;
 use thiserror::Error;
 use tracing::{debug, instrument, trace};
 use uv_cache::Cache;
+use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
 use uv_warnings::warn_user_once;
 use which::which;
 
-use crate::implementation::ImplementationName;
+use crate::implementation::{ImplementationName, LenientImplementationName};
 use crate::interpreter::Error as InterpreterError;
-use crate::managed::toolchains_for_current_platform;
+use crate::managed::InstalledToolchains;
 use crate::py_launcher::py_list_paths;
 use crate::virtualenv::{
     conda_prefix_from_env, virtualenv_from_env, virtualenv_from_working_dir,
@@ -26,8 +27,11 @@ use std::{path::Path, path::PathBuf, str::FromStr};
 /// A request to find a Python interpreter.
 ///
 /// See [`InterpreterRequest::from_str`].
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
 pub enum InterpreterRequest {
+    /// Use any discovered Python interpreter
+    #[default]
+    Any,
     /// A Python version without an implementation name e.g. `3.10`
     Version(VersionRequest),
     /// A path to a directory containing a Python installation, e.g. `.venv`
@@ -43,13 +47,12 @@ pub enum InterpreterRequest {
 }
 
 /// The sources to consider when finding a Python interpreter.
-#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum SourceSelector {
     // Consider all interpreter sources.
-    #[default]
-    All,
+    All(PreviewMode),
     // Only consider system interpreter sources
-    System,
+    System(PreviewMode),
     // Only consider virtual environment sources
     VirtualEnv,
     // Only consider a custom set of sources
@@ -60,7 +63,7 @@ pub enum SourceSelector {
 #[derive(Copy, Clone, Debug, Default, PartialEq, Eq)]
 pub enum VersionRequest {
     #[default]
-    Default,
+    Any,
     Major(u8),
     MajorMinor(u8, u8),
     MajorMinorPatch(u8, u8, u8),
@@ -235,17 +238,20 @@ fn python_executables<'a>(
     .chain(
         sources.contains(InterpreterSource::ManagedToolchain).then(move ||
             std::iter::once(
-                toolchains_for_current_platform()
-                .map(|toolchains|
+                InstalledToolchains::from_settings().map_err(Error::from).and_then(|installed_toolchains| {
+                    debug!("Searching for managed toolchains at `{}`", installed_toolchains.root().user_display());
+                    let toolchains = installed_toolchains.find_matching_current_platform()?;
                     // Check that the toolchain version satisfies the request to avoid unnecessary interpreter queries later
-                    toolchains.filter(move |toolchain|
-                        version.is_none() || version.is_some_and(|version|
-                            version.matches_version(toolchain.python_version())
+                    Ok(
+                        toolchains.into_iter().filter(move |toolchain|
+                            version.is_none() || version.is_some_and(|version|
+                                version.matches_version(toolchain.python_version())
+                            )
                         )
+                        .inspect(|toolchain| debug!("Found managed toolchain `{toolchain}`"))
+                        .map(|toolchain| (InterpreterSource::ManagedToolchain, toolchain.executable()))
                     )
-                    .map(|toolchain| (InterpreterSource::ManagedToolchain, toolchain.executable()))
-                )
-                .map_err(Error::from)
+                })
             ).flatten_ok()
         ).into_iter().flatten()
     )
@@ -297,7 +303,7 @@ fn python_executables_from_search_path<'a>(
         env::var_os("UV_TEST_PYTHON_PATH").unwrap_or(env::var_os("PATH").unwrap_or_default());
 
     let possible_names: Vec<_> = version
-        .unwrap_or(&VersionRequest::Default)
+        .unwrap_or(&VersionRequest::Any)
         .possible_names(implementation)
         .collect();
 
@@ -361,9 +367,9 @@ fn python_interpreters<'a>(
             Ok((source, path)) => Interpreter::query(&path, cache)
                 .map(|interpreter| (source, interpreter))
                 .inspect(|(source, interpreter)| {
-                    trace!(
-                        "Found Python interpreter {} {} at {} from {source}",
-                        interpreter.implementation_name(),
+                    debug!(
+                        "Found {} {} at `{}` ({source})",
+                        LenientImplementationName::from(interpreter.implementation_name()),
                         interpreter.python_full_version(),
                         path.display()
                     );
@@ -392,7 +398,7 @@ fn python_interpreters<'a>(
                         true
                     } else {
                         debug!(
-                            "Ignoring Python interpreter at `{}`: system intepreter not explicit",
+                            "Ignoring Python interpreter at `{}`: system interpreter not explicit",
                             interpreter.sys_executable().display()
                         );
                         false
@@ -401,7 +407,7 @@ fn python_interpreters<'a>(
                 (SystemPython::Explicit, true) => true,
                 (SystemPython::Disallowed, false) => {
                     debug!(
-                        "Ignoring Python interpreter at `{}`: system intepreter not allowed",
+                        "Ignoring Python interpreter at `{}`: system interpreter not allowed",
                         interpreter.sys_executable().display()
                     );
                     false
@@ -409,7 +415,7 @@ fn python_interpreters<'a>(
                 (SystemPython::Disallowed, true) => true,
                 (SystemPython::Required, true) => {
                     debug!(
-                        "Ignoring Python interpreter at `{}`: system intepreter required",
+                        "Ignoring Python interpreter at `{}`: system interpreter required",
                         interpreter.sys_executable().display()
                     );
                     false
@@ -423,7 +429,7 @@ fn python_interpreters<'a>(
 
 /// Check if an encountered error should stop discovery.
 ///
-/// Returns false when an error could be due to a faulty intepreter and we should continue searching for a working one.
+/// Returns false when an error could be due to a faulty interpreter and we should continue searching for a working one.
 fn should_stop_discovery(err: &Error) -> bool {
     match err {
         // When querying the interpreter fails, we will only raise errors that demonstrate that something is broken
@@ -453,9 +459,9 @@ pub fn find_interpreter(
     sources: &SourceSelector,
     cache: &Cache,
 ) -> Result<InterpreterResult, Error> {
-    debug!("Searching for interpreter that fulfills {request}");
     let result = match request {
         InterpreterRequest::File(path) => {
+            debug!("Checking for Python interpreter at {request}");
             if !sources.contains(InterpreterSource::ProvidedPath) {
                 return Err(Error::SourceNotSelected(
                     request.clone(),
@@ -473,6 +479,7 @@ pub fn find_interpreter(
             }
         }
         InterpreterRequest::Directory(path) => {
+            debug!("Checking for Python interpreter in {request}");
             if !sources.contains(InterpreterSource::ProvidedPath) {
                 return Err(Error::SourceNotSelected(
                     request.clone(),
@@ -496,6 +503,7 @@ pub fn find_interpreter(
             }
         }
         InterpreterRequest::ExecutableName(name) => {
+            debug!("Searching for Python interpreter with {request}");
             if !sources.contains(InterpreterSource::SearchPath) {
                 return Err(Error::SourceNotSelected(
                     request.clone(),
@@ -513,6 +521,7 @@ pub fn find_interpreter(
             }
         }
         InterpreterRequest::Implementation(implementation) => {
+            debug!("Searching for a {request} interpreter in {sources}");
             let Some((source, interpreter)) =
                 python_interpreters(None, Some(implementation), system, sources, cache)
                     .find(|result| {
@@ -536,6 +545,7 @@ pub fn find_interpreter(
             }
         }
         InterpreterRequest::ImplementationVersion(implementation, version) => {
+            debug!("Searching for {request} in {sources}");
             let Some((source, interpreter)) =
                 python_interpreters(Some(version), Some(implementation), system, sources, cache)
                     .find(|result| {
@@ -565,7 +575,30 @@ pub fn find_interpreter(
                 interpreter,
             }
         }
+        InterpreterRequest::Any => {
+            debug!("Searching for Python interpreter in {sources}");
+            let Some((source, interpreter)) =
+                python_interpreters(None, None, system, sources, cache)
+                    .find(|result| {
+                        match result {
+                            // Return the first critical error or interpreter
+                            Err(err) => should_stop_discovery(err),
+                            Ok(_) => true,
+                        }
+                    })
+                    .transpose()?
+            else {
+                return Ok(InterpreterResult::Err(
+                    InterpreterNotFound::NoPythonInstallation(sources.clone(), None),
+                ));
+            };
+            DiscoveredInterpreter {
+                source,
+                interpreter,
+            }
+        }
         InterpreterRequest::Version(version) => {
+            debug!("Searching for {request} in {sources}");
             let Some((source, interpreter)) =
                 python_interpreters(Some(version), None, system, sources, cache)
                     .find(|result| {
@@ -577,7 +610,7 @@ pub fn find_interpreter(
                     })
                     .transpose()?
             else {
-                let err = if matches!(version, VersionRequest::Default) {
+                let err = if matches!(version, VersionRequest::Any) {
                     InterpreterNotFound::NoPythonInstallation(sources.clone(), Some(*version))
                 } else {
                     InterpreterNotFound::NoMatchingVersion(sources.clone(), *version)
@@ -599,9 +632,12 @@ pub fn find_interpreter(
 /// Virtual environments are not included in discovery.
 ///
 /// See [`find_interpreter`] for more details on interpreter discovery.
-pub fn find_default_interpreter(cache: &Cache) -> Result<InterpreterResult, Error> {
-    let request = InterpreterRequest::Version(VersionRequest::Default);
-    let sources = SourceSelector::System;
+pub fn find_default_interpreter(
+    preview: PreviewMode,
+    cache: &Cache,
+) -> Result<InterpreterResult, Error> {
+    let request = InterpreterRequest::default();
+    let sources = SourceSelector::System(preview);
 
     let result = find_interpreter(&request, SystemPython::Required, &sources, cache)?;
     if let Ok(ref found) = result {
@@ -621,16 +657,17 @@ pub fn find_default_interpreter(cache: &Cache) -> Result<InterpreterResult, Erro
 /// the first available version.
 ///
 /// See [`find_interpreter`] for more details on interpreter discovery.
-#[instrument(skip_all, fields(?request))]
+#[instrument(skip_all, fields(request))]
 pub fn find_best_interpreter(
     request: &InterpreterRequest,
     system: SystemPython,
+    preview: PreviewMode,
     cache: &Cache,
 ) -> Result<InterpreterResult, Error> {
     debug!("Starting interpreter discovery for {}", request);
 
     // Determine if we should be allowed to look outside of virtual environments.
-    let sources = SourceSelector::from_settings(system);
+    let sources = SourceSelector::from_settings(system, preview);
 
     // First, check for an exact match (or the first available version if no Python versfion was provided)
     debug!("Looking for exact match for request {request}");
@@ -665,7 +702,7 @@ pub fn find_best_interpreter(
 
     // If a Python version was requested but cannot be fulfilled, just take any version
     debug!("Looking for Python interpreter with any version");
-    let request = InterpreterRequest::Version(VersionRequest::Default);
+    let request = InterpreterRequest::Any;
     Ok(find_interpreter(
         // TODO(zanieb): Add a dedicated `Default` variant to `InterpreterRequest`
         &request, system, &sources, cache,
@@ -917,7 +954,7 @@ impl VersionRequest {
         };
 
         match self {
-            Self::Default => [Some(python3), Some(python), None, None],
+            Self::Any => [Some(python3), Some(python), None, None],
             Self::Major(major) => [
                 Some(Cow::Owned(format!("python{major}{extension}"))),
                 Some(python),
@@ -960,7 +997,7 @@ impl VersionRequest {
                 };
 
                 match self {
-                    Self::Default => [Some(python3), Some(python), None, None],
+                    Self::Any => [Some(python3), Some(python), None, None],
                     Self::Major(major) => [
                         Some(Cow::Owned(format!("{name}{major}{extension}"))),
                         Some(python),
@@ -990,7 +1027,7 @@ impl VersionRequest {
     /// Check if a interpreter matches the requested Python version.
     fn matches_interpreter(self, interpreter: &Interpreter) -> bool {
         match self {
-            Self::Default => true,
+            Self::Any => true,
             Self::Major(major) => interpreter.python_major() == major,
             Self::MajorMinor(major, minor) => {
                 (interpreter.python_major(), interpreter.python_minor()) == (major, minor)
@@ -1007,7 +1044,7 @@ impl VersionRequest {
 
     fn matches_version(self, version: &PythonVersion) -> bool {
         match self {
-            Self::Default => true,
+            Self::Any => true,
             Self::Major(major) => version.major() == major,
             Self::MajorMinor(major, minor) => (version.major(), version.minor()) == (major, minor),
             Self::MajorMinorPatch(major, minor, patch) => {
@@ -1018,7 +1055,7 @@ impl VersionRequest {
 
     fn matches_major_minor(self, major: u8, minor: u8) -> bool {
         match self {
-            Self::Default => true,
+            Self::Any => true,
             Self::Major(self_major) => self_major == major,
             Self::MajorMinor(self_major, self_minor) => (self_major, self_minor) == (major, minor),
             Self::MajorMinorPatch(self_major, self_minor, _) => {
@@ -1030,7 +1067,7 @@ impl VersionRequest {
     /// Return true if a patch version is present in the request.
     fn has_patch(self) -> bool {
         match self {
-            Self::Default => false,
+            Self::Any => false,
             Self::Major(..) => false,
             Self::MajorMinor(..) => false,
             Self::MajorMinorPatch(..) => true,
@@ -1041,7 +1078,7 @@ impl VersionRequest {
     #[must_use]
     fn without_patch(self) -> Self {
         match self {
-            Self::Default => Self::Default,
+            Self::Any => Self::Any,
             Self::Major(major) => Self::Major(major),
             Self::MajorMinor(major, minor) => Self::MajorMinor(major, minor),
             Self::MajorMinorPatch(major, minor, _) => Self::MajorMinor(major, minor),
@@ -1082,7 +1119,7 @@ impl From<&PythonVersion> for VersionRequest {
 impl fmt::Display for VersionRequest {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
-            Self::Default => f.write_str("default"),
+            Self::Any => f.write_str("default"),
             Self::Major(major) => write!(f, "{major}"),
             Self::MajorMinor(major, minor) => write!(f, "{major}.{minor}"),
             Self::MajorMinorPatch(major, minor, patch) => {
@@ -1103,16 +1140,23 @@ impl SourceSelector {
     /// Return true if this selector includes the given [`InterpreterSource`].
     fn contains(&self, source: InterpreterSource) -> bool {
         match self {
-            Self::All => true,
-            Self::System => [
-                InterpreterSource::ProvidedPath,
-                InterpreterSource::SearchPath,
-                #[cfg(windows)]
-                InterpreterSource::PyLauncher,
-                InterpreterSource::ManagedToolchain,
-                InterpreterSource::ParentInterpreter,
-            ]
-            .contains(&source),
+            Self::All(preview) => {
+                // Always return `true` except for `ManagedToolchain` which requires preview mode
+                source != InterpreterSource::ManagedToolchain || preview.is_enabled()
+            }
+            Self::System(preview) => {
+                [
+                    InterpreterSource::ProvidedPath,
+                    InterpreterSource::SearchPath,
+                    #[cfg(windows)]
+                    InterpreterSource::PyLauncher,
+                    InterpreterSource::ParentInterpreter,
+                ]
+                .contains(&source)
+                    // Allow `ManagedToolchain` in preview
+                    || (source == InterpreterSource::ManagedToolchain
+                        && preview.is_enabled())
+            }
             Self::VirtualEnv => [
                 InterpreterSource::DiscoveredEnvironment,
                 InterpreterSource::ActiveEnvironment,
@@ -1124,7 +1168,7 @@ impl SourceSelector {
     }
 
     /// Return a [`SourceSelector`] based the settings.
-    pub fn from_settings(system: SystemPython) -> Self {
+    pub fn from_settings(system: SystemPython, preview: PreviewMode) -> Self {
         if env::var_os("UV_FORCE_MANAGED_PYTHON").is_some() {
             debug!("Only considering managed toolchains due to `UV_FORCE_MANAGED_PYTHON`");
             Self::from_sources([InterpreterSource::ManagedToolchain])
@@ -1138,8 +1182,8 @@ impl SourceSelector {
             ])
         } else {
             match system {
-                SystemPython::Allowed | SystemPython::Explicit => Self::All,
-                SystemPython::Required => Self::System,
+                SystemPython::Allowed | SystemPython::Explicit => Self::All(preview),
+                SystemPython::Required => Self::System(preview),
                 SystemPython::Disallowed => Self::VirtualEnv,
             }
         }
@@ -1161,15 +1205,16 @@ impl SystemPython {
 impl fmt::Display for InterpreterRequest {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
-            Self::Version(version) => write!(f, "Python @ {version}"),
-            Self::Directory(path) => write!(f, "directory {}", path.user_display()),
-            Self::File(path) => write!(f, "file {}", path.user_display()),
+            Self::Any => write!(f, "any Python"),
+            Self::Version(version) => write!(f, "Python {version}"),
+            Self::Directory(path) => write!(f, "directory `{}`", path.user_display()),
+            Self::File(path) => write!(f, "path `{}`", path.user_display()),
             Self::ExecutableName(name) => write!(f, "executable name `{name}`"),
             Self::Implementation(implementation) => {
                 write!(f, "{implementation}")
             }
             Self::ImplementationVersion(implementation, version) => {
-                write!(f, "{implementation} @ {version}")
+                write!(f, "{implementation} {version}")
             }
         }
     }
@@ -1193,13 +1238,13 @@ impl fmt::Display for InterpreterSource {
 impl fmt::Display for InterpreterNotFound {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
-            Self::NoPythonInstallation(sources, None | Some(VersionRequest::Default)) => {
+            Self::NoPythonInstallation(sources, None | Some(VersionRequest::Any)) => {
                 write!(f, "No Python interpreters found in {sources}")
             }
             Self::NoPythonInstallation(sources, Some(version)) => {
                 write!(f, "No Python {version} interpreters found in {sources}")
             }
-            Self::NoMatchingVersion(sources, VersionRequest::Default) => {
+            Self::NoMatchingVersion(sources, VersionRequest::Any) => {
                 write!(f, "No Python interpreter found in {sources}")
             }
             Self::NoMatchingVersion(sources, version) => {
@@ -1225,17 +1270,11 @@ impl fmt::Display for InterpreterNotFound {
                 path.user_display()
             ),
             Self::ExecutableNotFoundInDirectory(directory, executable) => {
-                let executable = if let Ok(relative_executable) = executable.strip_prefix(directory)
-                {
-                    relative_executable.display()
-                } else {
-                    executable.user_display()
-                };
                 write!(
                     f,
                     "Interpreter directory `{}` does not contain Python executable at `{}`",
                     directory.user_display(),
-                    executable
+                    executable.user_display_from(directory)
                 )
             }
             Self::ExecutableNotFoundInSearchPath(name) => {
@@ -1255,19 +1294,37 @@ impl fmt::Display for InterpreterNotFound {
 impl fmt::Display for SourceSelector {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         match self {
-            Self::All => f.write_str("all sources"),
+            Self::All(_) => f.write_str("all sources"),
             Self::VirtualEnv => f.write_str("virtual environments"),
-            Self::System => {
-                // TODO(zanieb): We intentionally omit managed toolchains for now since they are not public
+            Self::System(preview) => {
                 if cfg!(windows) {
-                    write!(
-                        f,
-                        "{} or {}",
-                        InterpreterSource::SearchPath,
-                        InterpreterSource::PyLauncher
-                    )
+                    if preview.is_disabled() {
+                        write!(
+                            f,
+                            "{} or {}",
+                            InterpreterSource::SearchPath,
+                            InterpreterSource::PyLauncher
+                        )
+                    } else {
+                        write!(
+                            f,
+                            "{}, {}, or {}",
+                            InterpreterSource::SearchPath,
+                            InterpreterSource::PyLauncher,
+                            InterpreterSource::ManagedToolchain
+                        )
+                    }
                 } else {
-                    write!(f, "{}", InterpreterSource::SearchPath)
+                    if preview.is_disabled() {
+                        write!(f, "{}", InterpreterSource::SearchPath)
+                    } else {
+                        write!(
+                            f,
+                            "{} or {}",
+                            InterpreterSource::SearchPath,
+                            InterpreterSource::ManagedToolchain
+                        )
+                    }
                 }
             }
             Self::Custom(sources) => {
