@@ -4,6 +4,7 @@ use futures::stream::FuturesUnordered;
 use futures::StreamExt;
 use rustc_hash::FxHashSet;
 use thiserror::Error;
+use tracing::trace;
 
 use distribution_types::{
     BuiltDist, Dist, DistributionMetadata, GitSourceDist, Requirement, RequirementSource,
@@ -13,7 +14,7 @@ use pep508_rs::MarkerEnvironment;
 use uv_configuration::{Constraints, Overrides};
 use uv_distribution::{DistributionDatabase, Reporter};
 use uv_git::GitUrl;
-use uv_resolver::{BuiltEditableMetadata, InMemoryIndex, MetadataResponse};
+use uv_resolver::{InMemoryIndex, MetadataResponse};
 use uv_types::{BuildContext, HashStrategy, RequestedRequirements};
 
 #[derive(Debug, Error)]
@@ -24,8 +25,6 @@ pub enum LookaheadError {
     DownloadAndBuild(SourceDist, #[source] uv_distribution::Error),
     #[error(transparent)]
     UnsupportedUrl(#[from] distribution_types::Error),
-    #[error(transparent)]
-    InvalidRequirement(#[from] Box<distribution_types::ParsedUrlError>),
 }
 
 /// A resolver for resolving lookahead requirements from direct URLs.
@@ -51,8 +50,6 @@ pub struct LookaheadResolver<'a, Context: BuildContext> {
     constraints: &'a Constraints,
     /// The overrides for the project.
     overrides: &'a Overrides,
-    /// The editable requirements for the project.
-    editables: &'a [BuiltEditableMetadata],
     /// The required hashes for the project.
     hasher: &'a HashStrategy,
     /// The in-memory index for resolving dependencies.
@@ -68,7 +65,6 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
         requirements: &'a [Requirement],
         constraints: &'a Constraints,
         overrides: &'a Overrides,
-        editables: &'a [BuiltEditableMetadata],
         hasher: &'a HashStrategy,
         index: &'a InMemoryIndex,
         database: DistributionDatabase<'a, Context>,
@@ -77,7 +73,6 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
             requirements,
             constraints,
             overrides,
-            editables,
             hasher,
             index,
             database,
@@ -112,13 +107,6 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
             .constraints
             .apply(self.overrides.apply(self.requirements))
             .filter(|requirement| requirement.evaluate_markers(markers, &[]))
-            .chain(self.editables.iter().flat_map(|editable| {
-                self.constraints
-                    .apply(self.overrides.apply(&editable.requirements.dependencies))
-                    .filter(|requirement| {
-                        requirement.evaluate_markers(markers, &editable.built.extras)
-                    })
-            }))
             .cloned()
             .collect();
 
@@ -154,6 +142,7 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
         &self,
         requirement: Requirement,
     ) -> Result<Option<RequestedRequirements>, LookaheadError> {
+        trace!("Performing lookahead for {requirement}");
         // Determine whether the requirement represents a local distribution and convert to a
         // buildable distribution.
         let dist = match requirement.source {
@@ -162,7 +151,7 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
                 subdirectory,
                 location,
                 url,
-            } => Dist::from_http_url(requirement.name, location, subdirectory, url)?,
+            } => Dist::from_http_url(requirement.name, url, location, subdirectory)?,
             RequirementSource::Git {
                 repository,
                 reference,
@@ -184,9 +173,8 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
             RequirementSource::Path {
                 path,
                 url,
-                // TODO(konsti): Figure out why we lose the editable here (does it matter?)
-                editable: _,
-            } => Dist::from_file_url(requirement.name, &path, false, url)?,
+                editable,
+            } => Dist::from_file_url(requirement.name, url, &path, editable)?,
         };
 
         // Fetch the metadata for the distribution.
@@ -211,8 +199,8 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
                     .requires_dist
                     .iter()
                     .cloned()
-                    .map(Requirement::from_pep508)
-                    .collect::<Result<_, _>>()?
+                    .map(Requirement::from)
+                    .collect()
             } else {
                 // Run the PEP 517 build process to extract metadata from the source distribution.
                 let archive = self
@@ -233,10 +221,7 @@ impl<'a, Context: BuildContext> LookaheadResolver<'a, Context> {
                     .distributions()
                     .done(id, Arc::new(MetadataResponse::Found(archive)));
 
-                requires_dist
-                    .into_iter()
-                    .map(Requirement::from_pep508)
-                    .collect::<Result<_, _>>()?
+                requires_dist.into_iter().map(Requirement::from).collect()
             }
         };
 
