@@ -1,5 +1,6 @@
 use std::borrow::Cow;
 use std::path::{Path, PathBuf};
+use std::sync::Arc;
 
 use anyhow::{Context, Result};
 use futures::stream::FuturesOrdered;
@@ -10,6 +11,7 @@ use distribution_types::{
     BuildableSource, DirectorySourceUrl, HashPolicy, Requirement, SourceUrl, VersionId,
 };
 use pep508_rs::RequirementOrigin;
+use pypi_types::VerbatimParsedUrl;
 use uv_distribution::{DistributionDatabase, Reporter};
 use uv_fs::Simplified;
 use uv_resolver::{InMemoryIndex, MetadataResponse};
@@ -73,12 +75,15 @@ impl<'a, Context: BuildContext> SourceTreeResolver<'a, Context> {
         Ok(requirements
             .into_iter()
             .flatten()
-            .map(Requirement::from_pep508)
-            .collect::<Result<_, _>>()?)
+            .map(Requirement::from)
+            .collect())
     }
 
     /// Infer the package name for a given "unnamed" requirement.
-    async fn resolve_source_tree(&self, path: &Path) -> Result<Vec<pep508_rs::Requirement>> {
+    async fn resolve_source_tree(
+        &self,
+        path: &Path,
+    ) -> Result<Vec<pep508_rs::Requirement<VerbatimParsedUrl>>> {
         // Convert to a buildable source.
         let source_tree = fs_err::canonicalize(path).with_context(|| {
             format!(
@@ -99,6 +104,7 @@ impl<'a, Context: BuildContext> SourceTreeResolver<'a, Context> {
         let source = SourceUrl::Directory(DirectorySourceUrl {
             url: &url,
             path: Cow::Borrowed(source_tree),
+            editable: false,
         });
 
         // Determine the hash policy. Since we don't have a package name, we perform a
@@ -117,17 +123,18 @@ impl<'a, Context: BuildContext> SourceTreeResolver<'a, Context> {
         // Fetch the metadata for the distribution.
         let metadata = {
             let id = VersionId::from_url(source.url());
-            if let Some(archive) = self
-                .index
-                .get_metadata(&id)
-                .as_deref()
-                .and_then(|response| {
-                    if let MetadataResponse::Found(archive) = response {
-                        Some(archive)
-                    } else {
-                        None
-                    }
-                })
+            if let Some(archive) =
+                self.index
+                    .distributions()
+                    .get(&id)
+                    .as_deref()
+                    .and_then(|response| {
+                        if let MetadataResponse::Found(archive) = response {
+                            Some(archive)
+                        } else {
+                            None
+                        }
+                    })
             {
                 // If the metadata is already in the index, return it.
                 archive.metadata.clone()
@@ -138,7 +145,8 @@ impl<'a, Context: BuildContext> SourceTreeResolver<'a, Context> {
 
                 // Insert the metadata into the index.
                 self.index
-                    .insert_metadata(id, MetadataResponse::Found(archive.clone()));
+                    .distributions()
+                    .done(id, Arc::new(MetadataResponse::Found(archive.clone())));
 
                 archive.metadata
             }
