@@ -11,12 +11,12 @@ use tracing::debug;
 
 use distribution_filename::{SourceDistFilename, WheelFilename};
 use distribution_types::{
-    BuildableSource, DirectSourceUrl, DirectorySourceUrl, GitSourceUrl, ParsedGitUrl,
-    PathSourceUrl, RemoteSource, Requirement, SourceUrl, UnresolvedRequirement,
+    BuildableSource, DirectSourceUrl, DirectorySourceUrl, GitSourceUrl, PathSourceUrl,
+    RemoteSource, Requirement, SourceUrl, UnresolvedRequirement,
     UnresolvedRequirementSpecification, VersionId,
 };
-use pep508_rs::{Scheme, UnnamedRequirement, VersionOrUrl};
-use pypi_types::Metadata10;
+use pep508_rs::{UnnamedRequirement, VersionOrUrl};
+use pypi_types::{Metadata10, ParsedUrl, VerbatimParsedUrl};
 use uv_distribution::{DistributionDatabase, Reporter};
 use uv_normalize::PackageName;
 use uv_resolver::{InMemoryIndex, MetadataResponse};
@@ -72,9 +72,9 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
             .map(|entry| async {
                 match entry.requirement {
                     UnresolvedRequirement::Named(requirement) => Ok(requirement),
-                    UnresolvedRequirement::Unnamed(requirement) => Ok(Requirement::from_pep508(
+                    UnresolvedRequirement::Unnamed(requirement) => Ok(Requirement::from(
                         Self::resolve_requirement(requirement, hasher, index, &database).await?,
-                    )?),
+                    )),
                 }
             })
             .collect::<FuturesOrdered<_>>()
@@ -84,19 +84,19 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
 
     /// Infer the package name for a given "unnamed" requirement.
     async fn resolve_requirement(
-        requirement: UnnamedRequirement,
+        requirement: UnnamedRequirement<VerbatimParsedUrl>,
         hasher: &HashStrategy,
         index: &InMemoryIndex,
         database: &DistributionDatabase<'a, Context>,
-    ) -> Result<pep508_rs::Requirement> {
+    ) -> Result<pep508_rs::Requirement<VerbatimParsedUrl>> {
         // If the requirement is a wheel, extract the package name from the wheel filename.
         //
         // Ex) `anyio-4.3.0-py3-none-any.whl`
-        if Path::new(requirement.url.path())
+        if Path::new(requirement.url.verbatim.path())
             .extension()
             .is_some_and(|ext| ext.eq_ignore_ascii_case("whl"))
         {
-            let filename = WheelFilename::from_str(&requirement.url.filename()?)?;
+            let filename = WheelFilename::from_str(&requirement.url.verbatim.filename()?)?;
             return Ok(pep508_rs::Requirement {
                 name: filename.name,
                 extras: requirement.extras,
@@ -112,6 +112,7 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
         // Ex) `anyio-4.3.0.tar.gz`
         if let Some(filename) = requirement
             .url
+            .verbatim
             .filename()
             .ok()
             .and_then(|filename| SourceDistFilename::parsed_normalized_filename(&filename).ok())
@@ -125,27 +126,43 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
             });
         }
 
-        let source = match Scheme::parse(requirement.url.scheme()) {
-            Some(Scheme::File) => {
-                let path = requirement
-                    .url
-                    .to_file_path()
-                    .expect("URL to be a file path");
+        let source = match &requirement.url.parsed_url {
+            // If the path points to a directory, attempt to read the name from static metadata.
+            ParsedUrl::Path(parsed_path_url) if parsed_path_url.path.is_dir() => {
+                // Attempt to read a `PKG-INFO` from the directory.
+                if let Some(metadata) = fs_err::read(parsed_path_url.path.join("PKG-INFO"))
+                    .ok()
+                    .and_then(|contents| Metadata10::parse_pkg_info(&contents).ok())
+                {
+                    debug!(
+                        "Found PKG-INFO metadata for {path} ({name})",
+                        path = parsed_path_url.path.display(),
+                        name = metadata.name
+                    );
+                    return Ok(pep508_rs::Requirement {
+                        name: metadata.name,
+                        extras: requirement.extras,
+                        version_or_url: Some(VersionOrUrl::Url(requirement.url)),
+                        marker: requirement.marker,
+                        origin: requirement.origin,
+                    });
+                }
 
-                // If the path points to a directory, attempt to read the name from static metadata.
-                if path.is_dir() {
-                    // Attempt to read a `PKG-INFO` from the directory.
-                    if let Some(metadata) = fs_err::read(path.join("PKG-INFO"))
-                        .ok()
-                        .and_then(|contents| Metadata10::parse_pkg_info(&contents).ok())
-                    {
+                // Attempt to read a `pyproject.toml` file.
+                let project_path = parsed_path_url.path.join("pyproject.toml");
+                if let Some(pyproject) = fs_err::read_to_string(project_path)
+                    .ok()
+                    .and_then(|contents| toml::from_str::<PyProjectToml>(&contents).ok())
+                {
+                    // Read PEP 621 metadata from the `pyproject.toml`.
+                    if let Some(project) = pyproject.project {
                         debug!(
-                            "Found PKG-INFO metadata for {path} ({name})",
-                            path = path.display(),
-                            name = metadata.name
+                            "Found PEP 621 metadata for {path} in `pyproject.toml` ({name})",
+                            path = parsed_path_url.path.display(),
+                            name = project.name
                         );
                         return Ok(pep508_rs::Requirement {
-                            name: metadata.name,
+                            name: project.name,
                             extras: requirement.extras,
                             version_or_url: Some(VersionOrUrl::Url(requirement.url)),
                             marker: requirement.marker,
@@ -153,106 +170,76 @@ impl<'a, Context: BuildContext> NamedRequirementsResolver<'a, Context> {
                         });
                     }
 
-                    // Attempt to read a `pyproject.toml` file.
-                    let project_path = path.join("pyproject.toml");
-                    if let Some(pyproject) = fs_err::read_to_string(project_path)
-                        .ok()
-                        .and_then(|contents| toml::from_str::<PyProjectToml>(&contents).ok())
-                    {
-                        // Read PEP 621 metadata from the `pyproject.toml`.
-                        if let Some(project) = pyproject.project {
-                            debug!(
-                                "Found PEP 621 metadata for {path} in `pyproject.toml` ({name})",
-                                path = path.display(),
-                                name = project.name
-                            );
-                            return Ok(pep508_rs::Requirement {
-                                name: project.name,
-                                extras: requirement.extras,
-                                version_or_url: Some(VersionOrUrl::Url(requirement.url)),
-                                marker: requirement.marker,
-                                origin: requirement.origin,
-                            });
-                        }
-
-                        // Read Poetry-specific metadata from the `pyproject.toml`.
-                        if let Some(tool) = pyproject.tool {
-                            if let Some(poetry) = tool.poetry {
-                                if let Some(name) = poetry.name {
-                                    debug!(
-                                        "Found Poetry metadata for {path} in `pyproject.toml` ({name})",
-                                        path = path.display(),
-                                        name = name
-                                    );
-                                    return Ok(pep508_rs::Requirement {
-                                        name,
-                                        extras: requirement.extras,
-                                        version_or_url: Some(VersionOrUrl::Url(requirement.url)),
-                                        marker: requirement.marker,
-                                        origin: requirement.origin,
-                                    });
-                                }
+                    // Read Poetry-specific metadata from the `pyproject.toml`.
+                    if let Some(tool) = pyproject.tool {
+                        if let Some(poetry) = tool.poetry {
+                            if let Some(name) = poetry.name {
+                                debug!(
+                                    "Found Poetry metadata for {path} in `pyproject.toml` ({name})",
+                                    path = parsed_path_url.path.display(),
+                                    name = name
+                                );
+                                return Ok(pep508_rs::Requirement {
+                                    name,
+                                    extras: requirement.extras,
+                                    version_or_url: Some(VersionOrUrl::Url(requirement.url)),
+                                    marker: requirement.marker,
+                                    origin: requirement.origin,
+                                });
                             }
                         }
                     }
+                }
 
-                    // Attempt to read a `setup.cfg` from the directory.
-                    if let Some(setup_cfg) = fs_err::read_to_string(path.join("setup.cfg"))
+                // Attempt to read a `setup.cfg` from the directory.
+                if let Some(setup_cfg) =
+                    fs_err::read_to_string(parsed_path_url.path.join("setup.cfg"))
                         .ok()
                         .and_then(|contents| {
                             let mut ini = Ini::new_cs();
                             ini.set_multiline(true);
                             ini.read(contents).ok()
                         })
-                    {
-                        if let Some(section) = setup_cfg.get("metadata") {
-                            if let Some(Some(name)) = section.get("name") {
-                                if let Ok(name) = PackageName::from_str(name) {
-                                    debug!(
-                                        "Found setuptools metadata for {path} in `setup.cfg` ({name})",
-                                        path = path.display(),
-                                        name = name
-                                    );
-                                    return Ok(pep508_rs::Requirement {
-                                        name,
-                                        extras: requirement.extras,
-                                        version_or_url: Some(VersionOrUrl::Url(requirement.url)),
-                                        marker: requirement.marker,
-                                        origin: requirement.origin,
-                                    });
-                                }
+                {
+                    if let Some(section) = setup_cfg.get("metadata") {
+                        if let Some(Some(name)) = section.get("name") {
+                            if let Ok(name) = PackageName::from_str(name) {
+                                debug!(
+                                    "Found setuptools metadata for {path} in `setup.cfg` ({name})",
+                                    path = parsed_path_url.path.display(),
+                                    name = name
+                                );
+                                return Ok(pep508_rs::Requirement {
+                                    name,
+                                    extras: requirement.extras,
+                                    version_or_url: Some(VersionOrUrl::Url(requirement.url)),
+                                    marker: requirement.marker,
+                                    origin: requirement.origin,
+                                });
                             }
                         }
                     }
-
-                    SourceUrl::Directory(DirectorySourceUrl {
-                        url: &requirement.url,
-                        path: Cow::Owned(path),
-                    })
-                } else {
-                    SourceUrl::Path(PathSourceUrl {
-                        url: &requirement.url,
-                        path: Cow::Owned(path),
-                    })
                 }
-            }
-            Some(Scheme::Http | Scheme::Https) => SourceUrl::Direct(DirectSourceUrl {
-                url: &requirement.url,
-            }),
-            Some(Scheme::GitSsh | Scheme::GitHttps | Scheme::GitHttp) => {
-                let git = ParsedGitUrl::try_from(requirement.url.to_url())?;
-                SourceUrl::Git(GitSourceUrl {
-                    git: Cow::Owned(git.url),
-                    subdirectory: git.subdirectory.map(Cow::Owned),
-                    url: &requirement.url,
+
+                SourceUrl::Directory(DirectorySourceUrl {
+                    url: &requirement.url.verbatim,
+                    path: Cow::Borrowed(&parsed_path_url.path),
+                    editable: parsed_path_url.editable,
                 })
             }
-            _ => {
-                return Err(anyhow::anyhow!(
-                    "Unsupported scheme for unnamed requirement: {}",
-                    requirement.url
-                ));
-            }
+            // If it's not a directory, assume it's a file.
+            ParsedUrl::Path(parsed_path_url) => SourceUrl::Path(PathSourceUrl {
+                url: &requirement.url.verbatim,
+                path: Cow::Borrowed(&parsed_path_url.path),
+            }),
+            ParsedUrl::Archive(parsed_archive_url) => SourceUrl::Direct(DirectSourceUrl {
+                url: &parsed_archive_url.url,
+            }),
+            ParsedUrl::Git(parsed_git_url) => SourceUrl::Git(GitSourceUrl {
+                url: &requirement.url.verbatim,
+                git: &parsed_git_url.url,
+                subdirectory: parsed_git_url.subdirectory.as_deref(),
+            }),
         };
 
         // Fetch the metadata for the distribution.
