@@ -175,7 +175,7 @@ impl FlatIndex {
             TagCompatibility::Compatible(priority) => priority,
         };
 
-        // Check if hashes line up
+        // Check if hashes line up.
         let hash = if let HashPolicy::Validate(required) = hasher.get_package(&filename.name) {
             if hashes.is_empty() {
                 HashComparison::Missing
@@ -188,7 +188,10 @@ impl FlatIndex {
             HashComparison::Matched
         };
 
-        WheelCompatibility::Compatible(hash, priority)
+        // Break ties with the build tag.
+        let build_tag = filename.build_tag.clone();
+
+        WheelCompatibility::Compatible(hash, priority, build_tag)
     }
 
     /// Get the [`FlatDistributions`] for the given package name.
