@@ -10,7 +10,7 @@ use uv_configuration::{Constraints, Overrides};
 use uv_normalize::{ExtraName, PackageName};
 
 use crate::pubgrub::specifier::PubGrubSpecifier;
-use crate::pubgrub::PubGrubPackage;
+use crate::pubgrub::{PubGrubPackage, PubGrubPackageInner};
 use crate::resolver::{Locals, Urls};
 use crate::ResolveError;
 
@@ -49,11 +49,6 @@ impl PubGrubDependencies {
         Ok(Self(dependencies))
     }
 
-    /// Add a [`PubGrubPackage`] and [`PubGrubVersion`] range into the dependencies.
-    pub(crate) fn push(&mut self, package: PubGrubPackage, version: Range<Version>) {
-        self.0.push((package, version));
-    }
-
     /// Iterate over the dependencies.
     pub(crate) fn iter(&self) -> impl Iterator<Item = &(PubGrubPackage, Range<Version>)> {
         self.0.iter()
@@ -102,8 +97,8 @@ fn add_requirements(
         })) {
             let PubGrubRequirement { package, version } = result?;
 
-            match &package {
-                PubGrubPackage::Package(name, ..) => {
+            match &*package {
+                PubGrubPackageInner::Package { name, .. } => {
                     // Detect self-dependencies.
                     if source_name.is_some_and(|source_name| source_name == name) {
                         warn!("{name} has a dependency on itself");
@@ -112,7 +107,7 @@ fn add_requirements(
 
                     dependencies.push((package.clone(), version.clone()));
                 }
-                PubGrubPackage::Extra(name, extra, ..) => {
+                PubGrubPackageInner::Extra { name, extra, .. } => {
                     // Recursively add the dependencies of the current package (e.g., `black` depending on
                     // `black[colorama]`).
                     if source_name.is_some_and(|source_name| source_name == name) {
@@ -158,7 +153,7 @@ fn add_requirements(
                     PubGrubRequirement::from_constraint(constraint, urls, locals)?;
 
                 // Ignore self-dependencies.
-                if let PubGrubPackage::Package(name, ..) = &package {
+                if let PubGrubPackageInner::Package { name, .. } = &*package {
                     // Detect self-dependencies.
                     if source_name.is_some_and(|source_name| source_name == name) {
                         warn!("{name} has a dependency on itself");
@@ -223,7 +218,12 @@ impl PubGrubRequirement {
                 };
 
                 Ok(Self {
-                    package: PubGrubPackage::from_package(requirement.name.clone(), extra, urls),
+                    package: PubGrubPackage::from_package(
+                        requirement.name.clone(),
+                        extra,
+                        None,
+                        urls,
+                    ),
                     version,
                 })
             }
@@ -244,11 +244,12 @@ impl PubGrubRequirement {
                 }
 
                 Ok(Self {
-                    package: PubGrubPackage::Package(
-                        requirement.name.clone(),
+                    package: PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: requirement.name.clone(),
                         extra,
-                        Some(expected.clone()),
-                    ),
+                        marker: None,
+                        url: Some(expected.clone()),
+                    }),
                     version: Range::full(),
                 })
             }
@@ -269,11 +270,12 @@ impl PubGrubRequirement {
                 }
 
                 Ok(Self {
-                    package: PubGrubPackage::Package(
-                        requirement.name.clone(),
+                    package: PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: requirement.name.clone(),
                         extra,
-                        Some(expected.clone()),
-                    ),
+                        marker: None,
+                        url: Some(expected.clone()),
+                    }),
                     version: Range::full(),
                 })
             }
@@ -294,11 +296,12 @@ impl PubGrubRequirement {
                 }
 
                 Ok(Self {
-                    package: PubGrubPackage::Package(
-                        requirement.name.clone(),
+                    package: PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: requirement.name.clone(),
                         extra,
-                        Some(expected.clone()),
-                    ),
+                        marker: None,
+                        url: Some(expected.clone()),
+                    }),
                     version: Range::full(),
                 })
             }
