@@ -7,16 +7,14 @@ use pubgrub::type_aliases::SelectedDependencies;
 use rustc_hash::{FxHashMap, FxHashSet};
 
 use distribution_types::{
-    Diagnostic, Dist, DistributionMetadata, Name, ParsedUrlError, Requirement, ResolvedDist,
-    VersionId, VersionOrUrlRef,
+    Dist, DistributionMetadata, Name, Requirement, ResolutionDiagnostic, VersionId, VersionOrUrlRef,
 };
 use pep440_rs::{Version, VersionSpecifier};
 use pep508_rs::MarkerEnvironment;
-use pypi_types::Yanked;
+use pypi_types::{ParsedUrlError, Yanked};
 use uv_normalize::PackageName;
 
 use crate::dependency_provider::UvDependencyProvider;
-use crate::editables::Editables;
 use crate::pins::FilePins;
 use crate::preferences::Preferences;
 use crate::pubgrub::{PubGrubDistribution, PubGrubPackageInner};
@@ -34,10 +32,8 @@ use crate::{
 pub struct ResolutionGraph {
     /// The underlying graph.
     pub(crate) petgraph: petgraph::graph::Graph<AnnotatedDist, Range<Version>, petgraph::Directed>,
-    /// The set of editable requirements in this resolution.
-    pub(crate) editables: Editables,
     /// Any diagnostics that were encountered while building the graph.
-    pub(crate) diagnostics: Vec<Diagnostic>,
+    pub(crate) diagnostics: Vec<ResolutionDiagnostic>,
 }
 
 impl ResolutionGraph {
@@ -50,7 +46,6 @@ impl ResolutionGraph {
         distributions: &FxOnceMap<VersionId, Arc<MetadataResponse>>,
         state: &State<UvDependencyProvider>,
         preferences: &Preferences,
-        editables: Editables,
     ) -> anyhow::Result<Self, ResolveError> {
         // Collect and validate the extras.
         let mut extras = FxHashMap::default();
@@ -90,7 +85,7 @@ impl ResolutionGraph {
                             .unwrap_or_else(|| panic!("Every package should be pinned: {name:?}"))
                             .clone();
 
-                        diagnostics.push(Diagnostic::MissingExtra {
+                        diagnostics.push(ResolutionDiagnostic::MissingExtra {
                             dist,
                             extra: extra.clone(),
                         });
@@ -102,50 +97,34 @@ impl ResolutionGraph {
                     marker: None,
                     url: Some(url),
                 } => {
-                    if let Some(editable) = editables.get(name) {
-                        if editable.metadata.provides_extras.contains(extra) {
-                            extras
-                                .entry(name.clone())
-                                .or_insert_with(Vec::new)
-                                .push(extra.clone());
-                        } else {
-                            let dist = Dist::from_editable(name.clone(), editable.built.clone())?;
+                    let dist = PubGrubDistribution::from_url(name, url);
 
-                            diagnostics.push(Diagnostic::MissingExtra {
-                                dist: dist.into(),
-                                extra: extra.clone(),
-                            });
-                        }
+                    let response = distributions.get(&dist.version_id()).unwrap_or_else(|| {
+                        panic!(
+                            "Every package should have metadata: {:?}",
+                            dist.version_id()
+                        )
+                    });
+
+                    let MetadataResponse::Found(archive) = &*response else {
+                        panic!(
+                            "Every package should have metadata: {:?}",
+                            dist.version_id()
+                        )
+                    };
+
+                    if archive.metadata.provides_extras.contains(extra) {
+                        extras
+                            .entry(name.clone())
+                            .or_insert_with(Vec::new)
+                            .push(extra.clone());
                     } else {
-                        let dist = PubGrubDistribution::from_url(name, url);
+                        let dist = Dist::from_url(name.clone(), url_to_precise(url.clone()))?;
 
-                        let response = distributions.get(&dist.version_id()).unwrap_or_else(|| {
-                            panic!(
-                                "Every package should have metadata: {:?}",
-                                dist.version_id()
-                            )
+                        diagnostics.push(ResolutionDiagnostic::MissingExtra {
+                            dist: dist.into(),
+                            extra: extra.clone(),
                         });
-
-                        let MetadataResponse::Found(archive) = &*response else {
-                            panic!(
-                                "Every package should have metadata: {:?}",
-                                dist.version_id()
-                            )
-                        };
-
-                        if archive.metadata.provides_extras.contains(extra) {
-                            extras
-                                .entry(name.clone())
-                                .or_insert_with(Vec::new)
-                                .push(extra.clone());
-                        } else {
-                            let dist = Dist::from_url(name.clone(), url_to_precise(url.clone()))?;
-
-                            diagnostics.push(Diagnostic::MissingExtra {
-                                dist: dist.into(),
-                                extra: extra.clone(),
-                            });
-                        }
                     }
                 }
                 _ => {}
@@ -177,13 +156,13 @@ impl ResolutionGraph {
                     match dist.yanked() {
                         None | Some(Yanked::Bool(false)) => {}
                         Some(Yanked::Bool(true)) => {
-                            diagnostics.push(Diagnostic::YankedVersion {
+                            diagnostics.push(ResolutionDiagnostic::YankedVersion {
                                 dist: dist.clone(),
                                 reason: None,
                             });
                         }
                         Some(Yanked::Reason(reason)) => {
-                            diagnostics.push(Diagnostic::YankedVersion {
+                            diagnostics.push(ResolutionDiagnostic::YankedVersion {
                                 dist: dist.clone(),
                                 reason: Some(reason.clone()),
                             });
@@ -254,75 +233,60 @@ impl ResolutionGraph {
                     url: Some(url),
                 } => {
                     // Create the distribution.
-                    if let Some(editable) = editables.get(name) {
-                        let dist = Dist::from_editable(name.clone(), editable.built.clone())?;
 
-                        // Add the distribution to the graph.
-                        let index = petgraph.add_node(AnnotatedDist {
-                            dist: dist.into(),
-                            extras: editable.built.extras.clone(),
-                            hashes: vec![],
-                            metadata: editable.metadata.clone(),
-                        });
-                        inverse.insert(name, index);
-                    } else {
-                        let dist = Dist::from_url(name.clone(), url_to_precise(url.clone()))?;
+                    let dist = Dist::from_url(name.clone(), url_to_precise(url.clone()))?;
 
-                        // Extract the hashes, preserving those that were already present in the
-                        // lockfile if necessary.
-                        let hashes = if let Some(digests) = preferences
-                            .match_hashes(name, version)
-                            .filter(|digests| !digests.is_empty())
-                        {
-                            digests.to_vec()
-                        } else if let Some(metadata_response) =
-                            distributions.get(&dist.version_id())
-                        {
-                            if let MetadataResponse::Found(ref archive) = *metadata_response {
-                                let mut digests = archive.hashes.clone();
-                                digests.sort_unstable();
-                                digests
-                            } else {
-                                vec![]
-                            }
+                    // Extract the hashes, preserving those that were already present in the
+                    // lockfile if necessary.
+                    let hashes = if let Some(digests) = preferences
+                        .match_hashes(name, version)
+                        .filter(|digests| !digests.is_empty())
+                    {
+                        digests.to_vec()
+                    } else if let Some(metadata_response) = distributions.get(&dist.version_id()) {
+                        if let MetadataResponse::Found(ref archive) = *metadata_response {
+                            let mut digests = archive.hashes.clone();
+                            digests.sort_unstable();
+                            digests
                         } else {
                             vec![]
-                        };
-
-                        // Extract the metadata.
-                        let metadata = {
-                            let dist = PubGrubDistribution::from_url(name, url);
-
-                            let response =
-                                distributions.get(&dist.version_id()).unwrap_or_else(|| {
-                                    panic!(
-                                        "Every package should have metadata: {:?}",
-                                        dist.version_id()
-                                    )
-                                });
-
-                            let MetadataResponse::Found(archive) = &*response else {
-                                panic!(
-                                    "Every package should have metadata: {:?}",
-                                    dist.version_id()
-                                )
-                            };
-
-                            archive.metadata.clone()
-                        };
-
-                        // Extract the extras.
-                        let extras = extras.get(name).cloned().unwrap_or_default();
-
-                        // Add the distribution to the graph.
-                        let index = petgraph.add_node(AnnotatedDist {
-                            dist: dist.into(),
-                            extras,
-                            hashes,
-                            metadata,
-                        });
-                        inverse.insert(name, index);
+                        }
+                    } else {
+                        vec![]
                     };
+
+                    // Extract the metadata.
+                    let metadata = {
+                        let dist = PubGrubDistribution::from_url(name, url);
+
+                        let response = distributions.get(&dist.version_id()).unwrap_or_else(|| {
+                            panic!(
+                                "Every package should have metadata: {:?}",
+                                dist.version_id()
+                            )
+                        });
+
+                        let MetadataResponse::Found(archive) = &*response else {
+                            panic!(
+                                "Every package should have metadata: {:?}",
+                                dist.version_id()
+                            )
+                        };
+
+                        archive.metadata.clone()
+                    };
+
+                    // Extract the extras.
+                    let extras = extras.get(name).cloned().unwrap_or_default();
+
+                    // Add the distribution to the graph.
+                    let index = petgraph.add_node(AnnotatedDist {
+                        dist: dist.into(),
+                        extras,
+                        hashes,
+                        metadata,
+                    });
+                    inverse.insert(name, index);
                 }
                 _ => {}
             };
@@ -380,7 +344,6 @@ impl ResolutionGraph {
 
         Ok(Self {
             petgraph,
-            editables,
             diagnostics,
         })
     }
@@ -402,17 +365,8 @@ impl ResolutionGraph {
             .any(|index| self.petgraph[index].name() == name)
     }
 
-    /// Iterate over the [`ResolvedDist`] entities in this resolution.
-    pub fn into_distributions(self) -> impl Iterator<Item = ResolvedDist> {
-        self.petgraph
-            .into_nodes_edges()
-            .0
-            .into_iter()
-            .map(|node| node.weight.dist)
-    }
-
-    /// Return the [`Diagnostic`]s that were encountered while building the graph.
-    pub fn diagnostics(&self) -> &[Diagnostic] {
+    /// Return the [`ResolutionDiagnostic`]s that were encountered while building the graph.
+    pub fn diagnostics(&self) -> &[ResolutionDiagnostic] {
         &self.diagnostics
     }
 
@@ -512,8 +466,8 @@ impl ResolutionGraph {
                 .requires_dist
                 .iter()
                 .cloned()
-                .map(Requirement::from_pep508)
-                .collect::<anyhow::Result<_, _>>()?;
+                .map(Requirement::from)
+                .collect();
             for req in manifest.apply(requirements.iter()) {
                 let Some(ref marker_tree) = req.marker else {
                     continue;
@@ -523,13 +477,7 @@ impl ResolutionGraph {
         }
 
         // Ensure that we consider markers from direct dependencies.
-        let direct_reqs = manifest.requirements.iter().chain(
-            manifest
-                .editables
-                .iter()
-                .flat_map(|editable| &editable.requirements.dependencies),
-        );
-        for direct_req in manifest.apply(direct_reqs) {
+        for direct_req in manifest.apply(manifest.requirements.iter()) {
             let Some(ref marker_tree) = direct_req.marker else {
                 continue;
             };
