@@ -37,17 +37,19 @@ use uv_types::{BuildContext, HashStrategy, InstalledPackagesProvider};
 
 use crate::candidate_selector::{CandidateDist, CandidateSelector};
 use crate::dependency_provider::UvDependencyProvider;
-use crate::editables::Editables;
 use crate::error::ResolveError;
 use crate::manifest::Manifest;
 use crate::pins::FilePins;
 use crate::preferences::Preferences;
 use crate::pubgrub::{
     PubGrubDependencies, PubGrubDistribution, PubGrubPackage, PubGrubPackageInner,
-    PubGrubPriorities, PubGrubPython, PubGrubRequirement, PubGrubSpecifier,
+    PubGrubPriorities, PubGrubPython, PubGrubSpecifier,
 };
 use crate::python_requirement::PythonRequirement;
 use crate::resolution::ResolutionGraph;
+pub(crate) use crate::resolver::availability::{
+    IncompletePackage, ResolverVersion, UnavailablePackage, UnavailableReason, UnavailableVersion,
+};
 use crate::resolver::batch_prefetch::BatchPrefetcher;
 pub(crate) use crate::resolver::index::FxOnceMap;
 pub use crate::resolver::index::InMemoryIndex;
@@ -60,6 +62,7 @@ pub use crate::resolver::reporter::{BuildId, Reporter};
 use crate::yanks::AllowedYanks;
 use crate::{DependencyMode, Exclusions, FlatIndex, Options};
 
+mod availability;
 mod batch_prefetch;
 mod index;
 mod locals;
@@ -67,113 +70,6 @@ mod provider;
 mod reporter;
 mod urls;
 
-/// The reason why a package or a version cannot be used.
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) enum UnavailableReason {
-    /// The entire package cannot be used.
-    Package(UnavailablePackage),
-    /// A single version cannot be used.
-    Version(UnavailableVersion),
-}
-
-impl Display for UnavailableReason {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            Self::Version(version) => Display::fmt(version, f),
-            Self::Package(package) => Display::fmt(package, f),
-        }
-    }
-}
-
-/// The package version is unavailable and cannot be used. Unlike [`PackageUnavailable`], this
-/// applies to a single version of the package.
-///
-/// Most variant are from [`MetadataResponse`] without the error source (since we don't format
-/// the source).
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) enum UnavailableVersion {
-    /// Version is incompatible because it has no usable distributions
-    IncompatibleDist(IncompatibleDist),
-    /// The wheel metadata was found, but could not be parsed.
-    InvalidMetadata,
-    /// The wheel metadata was found, but the metadata was inconsistent.
-    InconsistentMetadata,
-    /// The wheel has an invalid structure.
-    InvalidStructure,
-    /// The wheel metadata was not found in the cache and the network is not available.
-    Offline,
-    /// Forward any kind of resolver error.
-    ResolverError(String),
-}
-
-impl Display for UnavailableVersion {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        match self {
-            UnavailableVersion::IncompatibleDist(invalid_dist) => Display::fmt(invalid_dist, f),
-            UnavailableVersion::InvalidMetadata => f.write_str("has invalid metadata"),
-            UnavailableVersion::InconsistentMetadata => f.write_str("has inconsistent metadata"),
-            UnavailableVersion::InvalidStructure => f.write_str("has an invalid package format"),
-            UnavailableVersion::Offline => f.write_str(
-                "network connectivity is disabled, but the metadata wasn't found in the cache",
-            ),
-            UnavailableVersion::ResolverError(err) => f.write_str(err),
-        }
-    }
-}
-
-/// The package is unavailable and cannot be used.
-#[derive(Debug, Clone, Eq, PartialEq)]
-pub(crate) enum UnavailablePackage {
-    /// Index lookups were disabled (i.e., `--no-index`) and the package was not found in a flat index (i.e. from `--find-links`).
-    NoIndex,
-    /// Network requests were disabled (i.e., `--offline`), and the package was not found in the cache.
-    Offline,
-    /// The package was not found in the registry.
-    NotFound,
-    /// The package metadata was found, but could not be parsed.
-    InvalidMetadata(String),
-    /// The package has an invalid structure.
-    InvalidStructure(String),
-}
-
-impl UnavailablePackage {
-    pub(crate) fn as_str(&self) -> &'static str {
-        match self {
-            UnavailablePackage::NoIndex => "was not found in the provided package locations",
-            UnavailablePackage::Offline => "was not found in the cache",
-            UnavailablePackage::NotFound => "was not found in the package registry",
-            UnavailablePackage::InvalidMetadata(_) => "has invalid metadata",
-            UnavailablePackage::InvalidStructure(_) => "has an invalid package format",
-        }
-    }
-}
-
-impl Display for UnavailablePackage {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        f.write_str(self.as_str())
-    }
-}
-
-/// The package is unavailable at specific versions.
-#[derive(Debug, Clone)]
-pub(crate) enum IncompletePackage {
-    /// Network requests were disabled (i.e., `--offline`), and the wheel metadata was not found in the cache.
-    Offline,
-    /// The wheel metadata was found, but could not be parsed.
-    InvalidMetadata(String),
-    /// The wheel metadata was found, but the metadata was inconsistent.
-    InconsistentMetadata(String),
-    /// The wheel has an invalid structure.
-    InvalidStructure(String),
-}
-
-enum ResolverVersion {
-    /// A usable version
-    Available(Version),
-    /// A version that is not usable for some reason
-    Unavailable(Version, UnavailableVersion),
-}
-
 pub struct Resolver<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider> {
     state: ResolverState<InstalledPackages>,
     provider: Provider,
@@ -188,7 +84,6 @@ struct ResolverState<InstalledPackages: InstalledPackagesProvider> {
     overrides: Overrides,
     preferences: Preferences,
     exclusions: Exclusions,
-    editables: Editables,
     urls: Urls,
     locals: Locals,
     dependency_mode: DependencyMode,
@@ -295,7 +190,6 @@ impl<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider>
             overrides: manifest.overrides,
             preferences: Preferences::from_iter(manifest.preferences, markers),
             exclusions: manifest.exclusions,
-            editables: Editables::from_requirements(manifest.editables),
             hasher: hasher.clone(),
             markers: markers.cloned(),
             python_requirement: python_requirement.clone(),
@@ -446,7 +340,6 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     self.index.distributions(),
                     &state.pubgrub,
                     &self.preferences,
-                    self.editables.clone(),
                 );
             };
             state.next = highest_priority_pkg;
@@ -663,11 +556,6 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     return Err(ResolveError::UnhashedPackage(name.clone()));
                 }
 
-                // If the package is an editable, we don't need to fetch metadata.
-                if self.editables.contains(name) {
-                    return Ok(());
-                }
-
                 // Emit a request to fetch the metadata for this distribution.
                 let dist = Dist::from_url(name.clone(), url.clone())?;
                 if self.index.distributions().register(dist.version_id()) {
@@ -752,31 +640,6 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     url.verbatim
                 );
 
-                // If the dist is an editable, return the version from the editable metadata.
-                if let Some(editable) = self.editables.get(name) {
-                    let version = &editable.metadata.version;
-
-                    // The version is incompatible with the requirement.
-                    if !range.contains(version) {
-                        return Ok(None);
-                    }
-
-                    // The version is incompatible due to its Python requirement.
-                    if let Some(requires_python) = editable.metadata.requires_python.as_ref() {
-                        let target = self.python_requirement.target();
-                        if !requires_python.contains(target) {
-                            return Ok(Some(ResolverVersion::Unavailable(
-                                version.clone(),
-                                UnavailableVersion::IncompatibleDist(IncompatibleDist::Source(
-                                    IncompatibleSource::RequiresPython(requires_python.clone()),
-                                )),
-                            )));
-                        }
-                    }
-
-                    return Ok(Some(ResolverVersion::Available(version.clone())));
-                }
-
                 let dist = PubGrubDistribution::from_url(name, url);
                 let response = self
                     .index
@@ -956,7 +819,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     self.markers.as_ref(),
                 );
 
-                let mut dependencies = match dependencies {
+                let dependencies = match dependencies {
                     Ok(dependencies) => dependencies,
                     Err(err) => {
                         return Ok(Dependencies::Unavailable(
@@ -975,54 +838,6 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     self.visit_package(package, request_sink)?;
                 }
 
-                // Add a dependency on each editable.
-                for editable in self.editables.iter() {
-                    let package = PubGrubPackage::from_package(
-                        editable.metadata.name.clone(),
-                        None,
-                        None,
-                        &self.urls,
-                    );
-                    let version = Range::singleton(editable.metadata.version.clone());
-
-                    // Update the package priorities.
-                    priorities.insert(&package, &version);
-
-                    // Add the editable as a direct dependency.
-                    dependencies.push(package, version);
-
-                    // Add a dependency on each extra.
-                    for extra in &editable.built.extras {
-                        dependencies.push(
-                            PubGrubPackage::from_package(
-                                editable.metadata.name.clone(),
-                                Some(extra.clone()),
-                                None,
-                                &self.urls,
-                            ),
-                            Range::singleton(editable.metadata.version.clone()),
-                        );
-                    }
-
-                    // Add any constraints.
-                    for constraint in self
-                        .constraints
-                        .get(&editable.metadata.name)
-                        .into_iter()
-                        .flatten()
-                    {
-                        if constraint.evaluate_markers(self.markers.as_ref(), &[]) {
-                            let PubGrubRequirement { package, version } =
-                                PubGrubRequirement::from_constraint(
-                                    constraint,
-                                    &self.urls,
-                                    &self.locals,
-                                )?;
-                            dependencies.push(package, version);
-                        }
-                    }
-                }
-
                 Ok(Dependencies::Available(dependencies.into()))
             }
 
@@ -1038,57 +853,22 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 if self.dependency_mode.is_direct() {
                     // If an extra is provided, wait for the metadata to be available, since it's
                     // still required for generating the lock file.
-                    if !self.editables.contains(name) {
-                        // Determine the distribution to lookup.
-                        let dist = match url {
-                            Some(url) => PubGrubDistribution::from_url(name, url),
-                            None => PubGrubDistribution::from_registry(name, version),
-                        };
-                        let version_id = dist.version_id();
 
-                        // Wait for the metadata to be available.
-                        self.index
-                            .distributions()
-                            .wait_blocking(&version_id)
-                            .ok_or(ResolveError::Unregistered)?;
-                    }
+                    let dist = match url {
+                        Some(url) => PubGrubDistribution::from_url(name, url),
+                        None => PubGrubDistribution::from_registry(name, version),
+                    };
+                    let version_id = dist.version_id();
+
+                    // Wait for the metadata to be available.
+                    self.index
+                        .distributions()
+                        .wait_blocking(&version_id)
+                        .ok_or(ResolveError::Unregistered)?;
 
                     return Ok(Dependencies::Available(Vec::default()));
                 }
 
-                // Determine if the distribution is editable.
-                if let Some(editable) = self.editables.get(name) {
-                    let requirements: Vec<_> = editable
-                        .metadata
-                        .requires_dist
-                        .iter()
-                        .cloned()
-                        .map(Requirement::from_pep508)
-                        .collect::<Result<_, _>>()?;
-                    let dependencies = PubGrubDependencies::from_requirements(
-                        &requirements,
-                        &self.constraints,
-                        &self.overrides,
-                        Some(name),
-                        extra.as_ref(),
-                        &self.urls,
-                        &self.locals,
-                        self.markers.as_ref(),
-                    )?;
-
-                    for (dep_package, dep_version) in dependencies.iter() {
-                        debug!("Adding transitive dependency for {package}=={version}: {dep_package}{dep_version}");
-
-                        // Update the package priorities.
-                        priorities.insert(dep_package, dep_version);
-
-                        // Emit a request to fetch the metadata for this package.
-                        self.visit_package(dep_package, request_sink)?;
-                    }
-
-                    return Ok(Dependencies::Available(dependencies.into()));
-                }
-
                 // Determine the distribution to lookup.
                 let dist = match url {
                     Some(url) => PubGrubDistribution::from_url(name, url),
@@ -1170,8 +950,8 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     .requires_dist
                     .iter()
                     .cloned()
-                    .map(Requirement::from_pep508)
-                    .collect::<Result<_, _>>()?;
+                    .map(Requirement::from)
+                    .collect();
                 let dependencies = PubGrubDependencies::from_requirements(
                     &requirements,
                     &self.constraints,
@@ -1335,6 +1115,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                             ResolveError::FetchAndBuild(Box::new(source_dist), err)
                         }
                     })?;
+
                 Ok(Some(Response::Dist { dist, metadata }))
             }
 
@@ -1423,6 +1204,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                                         ResolveError::FetchAndBuild(Box::new(source_dist), err)
                                     }
                                 })?;
+
                             Response::Dist { dist, metadata }
                         }
                         ResolvedDist::Installed(dist) => {
