@@ -1,11 +1,9 @@
+use distribution_types::{RequirementSource, Verbatim};
 use rustc_hash::FxHashMap;
 use tracing::debug;
 
-use distribution_types::{
-    ParsedArchiveUrl, ParsedGitUrl, ParsedPathUrl, ParsedUrl, RequirementSource, Verbatim,
-    VerbatimParsedUrl,
-};
 use pep508_rs::{MarkerEnvironment, VerbatimUrl};
+use pypi_types::{ParsedArchiveUrl, ParsedGitUrl, ParsedPathUrl, ParsedUrl, VerbatimParsedUrl};
 use uv_distribution::is_same_reference;
 use uv_git::GitUrl;
 use uv_normalize::PackageName;
@@ -24,36 +22,6 @@ impl Urls {
     ) -> Result<Self, ResolveError> {
         let mut urls: FxHashMap<PackageName, VerbatimParsedUrl> = FxHashMap::default();
 
-        // Add the editables themselves to the list of required URLs.
-        for editable in &manifest.editables {
-            let editable_url = VerbatimParsedUrl {
-                parsed_url: ParsedUrl::Path(ParsedPathUrl {
-                    url: editable.built.url.to_url(),
-                    path: editable.built.path.clone(),
-                    editable: true,
-                }),
-                verbatim: editable.built.url.clone(),
-            };
-            if let Some(previous) =
-                urls.insert(editable.metadata.name.clone(), editable_url.clone())
-            {
-                if !is_equal(&previous.verbatim, &editable_url.verbatim) {
-                    if is_same_reference(&previous.verbatim, &editable_url.verbatim) {
-                        debug!(
-                            "Allowing {} as a variant of {}",
-                            editable_url.verbatim, previous.verbatim
-                        );
-                    } else {
-                        return Err(ResolveError::ConflictingUrlsDirect(
-                            editable.metadata.name.clone(),
-                            previous.verbatim.verbatim().to_string(),
-                            editable_url.verbatim.verbatim().to_string(),
-                        ));
-                    }
-                }
-            }
-        }
-
         // Add all direct requirements and constraints. If there are any conflicts, return an error.
         for requirement in manifest.requirements(markers, dependencies) {
             match &requirement.source {
