@@ -11,15 +11,17 @@ use chrono::{DateTime, Utc};
 use once_cell::sync::Lazy;
 
 use distribution_types::{CachedDist, IndexLocations, Requirement, Resolution, SourceDist};
+use pep440_rs::Version;
 use pep508_rs::{MarkerEnvironment, MarkerEnvironmentBuilder};
 use platform_tags::{Arch, Os, Platform, Tags};
 use uv_cache::Cache;
 use uv_client::RegistryClientBuilder;
 use uv_configuration::{
-    BuildKind, Concurrency, Constraints, NoBinary, NoBuild, Overrides, SetupPyStrategy,
+    BuildKind, Concurrency, Constraints, NoBinary, NoBuild, Overrides, PreviewMode, SetupPyStrategy,
 };
 use uv_distribution::DistributionDatabase;
 use uv_interpreter::{find_default_interpreter, Interpreter, PythonEnvironment};
+use uv_normalize::PackageName;
 use uv_resolver::{
     DisplayResolutionGraph, ExcludeNewer, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options,
     OptionsBuilder, PreReleaseMode, Preference, PythonRequirement, ResolutionGraph, ResolutionMode,
@@ -131,7 +133,7 @@ async fn resolve(
     let client = RegistryClientBuilder::new(cache).build();
     let flat_index = FlatIndex::default();
     let index = InMemoryIndex::default();
-    let real_interpreter = find_default_interpreter(&Cache::temp().unwrap())
+    let real_interpreter = find_default_interpreter(PreviewMode::Disabled, &Cache::temp().unwrap())
         .unwrap()
         .expect("Python should be installed")
         .into_interpreter();
@@ -167,10 +169,9 @@ macro_rules! assert_snapshot {
 
 #[tokio::test]
 async fn black() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -196,10 +197,9 @@ async fn black() -> Result<()> {
 
 #[tokio::test]
 async fn black_colorama() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black[colorama]<=23.9.1").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -228,10 +228,9 @@ async fn black_colorama() -> Result<()> {
 /// Resolve Black with an invalid extra. The resolver should ignore the extra.
 #[tokio::test]
 async fn black_tensorboard() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black[tensorboard]<=23.9.1").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -257,10 +256,9 @@ async fn black_tensorboard() -> Result<()> {
 
 #[tokio::test]
 async fn black_python_310() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
@@ -293,18 +291,15 @@ async fn black_python_310() -> Result<()> {
 #[tokio::test]
 async fn black_mypy_extensions() -> Result<()> {
     let manifest = Manifest::new(
-        vec![
-            Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1").unwrap())
-                .unwrap(),
-        ],
-        Constraints::from_requirements(vec![Requirement::from_pep508(
+        vec![Requirement::from(
+            pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
+        )],
+        Constraints::from_requirements(vec![Requirement::from(
             pep508_rs::Requirement::from_str("mypy-extensions<0.4.4").unwrap(),
-        )
-        .unwrap()]),
+        )]),
         Overrides::default(),
         vec![],
         None,
-        vec![],
         Exclusions::default(),
         vec![],
     );
@@ -336,18 +331,15 @@ async fn black_mypy_extensions() -> Result<()> {
 #[tokio::test]
 async fn black_mypy_extensions_extra() -> Result<()> {
     let manifest = Manifest::new(
-        vec![
-            Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1").unwrap())
-                .unwrap(),
-        ],
-        Constraints::from_requirements(vec![Requirement::from_pep508(
+        vec![Requirement::from(
+            pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
+        )],
+        Constraints::from_requirements(vec![Requirement::from(
             pep508_rs::Requirement::from_str("mypy-extensions[extra]<0.4.4").unwrap(),
-        )
-        .unwrap()]),
+        )]),
         Overrides::default(),
         vec![],
         None,
-        vec![],
         Exclusions::default(),
         vec![],
     );
@@ -379,18 +371,15 @@ async fn black_mypy_extensions_extra() -> Result<()> {
 #[tokio::test]
 async fn black_flake8() -> Result<()> {
     let manifest = Manifest::new(
-        vec![
-            Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1").unwrap())
-                .unwrap(),
-        ],
-        Constraints::from_requirements(vec![Requirement::from_pep508(
+        vec![Requirement::from(
+            pep508_rs::Requirement::from_str("black<=23.9.1").unwrap(),
+        )],
+        Constraints::from_requirements(vec![Requirement::from(
             pep508_rs::Requirement::from_str("flake8<1").unwrap(),
-        )
-        .unwrap()]),
+        )]),
         Overrides::default(),
         vec![],
         None,
-        vec![],
         Exclusions::default(),
         vec![],
     );
@@ -419,10 +408,9 @@ async fn black_flake8() -> Result<()> {
 
 #[tokio::test]
 async fn black_lowest() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black>21").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .resolution_mode(ResolutionMode::Lowest)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -449,10 +437,9 @@ async fn black_lowest() -> Result<()> {
 
 #[tokio::test]
 async fn black_lowest_direct() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black>21").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .resolution_mode(ResolutionMode::LowestDirect)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -480,14 +467,16 @@ async fn black_lowest_direct() -> Result<()> {
 #[tokio::test]
 async fn black_respect_preference() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1")?).unwrap()],
+        vec![Requirement::from(pep508_rs::Requirement::from_str(
+            "black<=23.9.1",
+        )?)],
         Constraints::default(),
         Overrides::default(),
-        vec![Preference::from_requirement(
-            Requirement::from_pep508(pep508_rs::Requirement::from_str("black==23.9.0")?).unwrap(),
+        vec![Preference::simple(
+            PackageName::from_str("black")?,
+            Version::from_str("23.9.0")?,
         )],
         None,
-        vec![],
         Exclusions::default(),
         vec![],
     );
@@ -518,14 +507,16 @@ async fn black_respect_preference() -> Result<()> {
 #[tokio::test]
 async fn black_ignore_preference() -> Result<()> {
     let manifest = Manifest::new(
-        vec![Requirement::from_pep508(pep508_rs::Requirement::from_str("black<=23.9.1")?).unwrap()],
+        vec![Requirement::from(pep508_rs::Requirement::from_str(
+            "black<=23.9.1",
+        )?)],
         Constraints::default(),
         Overrides::default(),
-        vec![Preference::from_requirement(
-            Requirement::from_pep508(pep508_rs::Requirement::from_str("black==23.9.2")?).unwrap(),
+        vec![Preference::simple(
+            PackageName::from_str("black")?,
+            Version::from_str("23.9.2")?,
         )],
         None,
-        vec![],
         Exclusions::default(),
         vec![],
     );
@@ -554,10 +545,9 @@ async fn black_ignore_preference() -> Result<()> {
 
 #[tokio::test]
 async fn black_disallow_prerelease() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black<=20.0").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Disallow)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -578,10 +568,9 @@ async fn black_disallow_prerelease() -> Result<()> {
 
 #[tokio::test]
 async fn black_allow_prerelease_if_necessary() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("black<=20.0").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::IfNecessary)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -602,10 +591,9 @@ async fn black_allow_prerelease_if_necessary() -> Result<()> {
 
 #[tokio::test]
 async fn pylint_disallow_prerelease() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Disallow)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -628,10 +616,9 @@ async fn pylint_disallow_prerelease() -> Result<()> {
 
 #[tokio::test]
 async fn pylint_allow_prerelease() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Allow)
         .exclude_newer(Some(*EXCLUDE_NEWER))
@@ -655,10 +642,8 @@ async fn pylint_allow_prerelease() -> Result<()> {
 #[tokio::test]
 async fn pylint_allow_explicit_prerelease_without_marker() -> Result<()> {
     let manifest = Manifest::simple(vec![
-        Requirement::from_pep508(pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap())
-            .unwrap(),
-        Requirement::from_pep508(pep508_rs::Requirement::from_str("isort>=5.0.0").unwrap())
-            .unwrap(),
+        Requirement::from(pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap()),
+        Requirement::from(pep508_rs::Requirement::from_str("isort>=5.0.0").unwrap()),
     ]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Explicit)
@@ -683,10 +668,8 @@ async fn pylint_allow_explicit_prerelease_without_marker() -> Result<()> {
 #[tokio::test]
 async fn pylint_allow_explicit_prerelease_with_marker() -> Result<()> {
     let manifest = Manifest::simple(vec![
-        Requirement::from_pep508(pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap())
-            .unwrap(),
-        Requirement::from_pep508(pep508_rs::Requirement::from_str("isort>=5.0.0b").unwrap())
-            .unwrap(),
+        Requirement::from(pep508_rs::Requirement::from_str("pylint==2.3.0").unwrap()),
+        Requirement::from(pep508_rs::Requirement::from_str("isort>=5.0.0b").unwrap()),
     ]);
     let options = OptionsBuilder::new()
         .prerelease_mode(PreReleaseMode::Explicit)
@@ -712,10 +695,9 @@ async fn pylint_allow_explicit_prerelease_with_marker() -> Result<()> {
 /// fail with a pre-release-centric hint.
 #[tokio::test]
 async fn msgraph_sdk() -> Result<()> {
-    let manifest = Manifest::simple(vec![Requirement::from_pep508(
+    let manifest = Manifest::simple(vec![Requirement::from(
         pep508_rs::Requirement::from_str("msgraph-sdk==1.0.0").unwrap(),
-    )
-    .unwrap()]);
+    )]);
     let options = OptionsBuilder::new()
         .exclude_newer(Some(*EXCLUDE_NEWER))
         .build();
