@@ -2,7 +2,6 @@ use std::future::Future;
 use std::path::{Path, PathBuf};
 
 use anyhow::Result;
-use url::Url;
 
 use distribution_types::{
     CachedDist, IndexLocations, InstalledDist, Requirement, Resolution, SourceDist,
@@ -134,7 +133,6 @@ pub trait SourceBuildTrait {
 pub trait InstalledPackagesProvider: Clone + Send + Sync + 'static {
     fn iter(&self) -> impl Iterator<Item = &InstalledDist>;
     fn get_packages(&self, name: &PackageName) -> Vec<&InstalledDist>;
-    fn get_editables(&self, url: &Url) -> Vec<&InstalledDist>;
 }
 
 /// An [`InstalledPackagesProvider`] with no packages in it.
@@ -149,8 +147,4 @@ impl InstalledPackagesProvider for EmptyInstalledPackages {
     fn iter(&self) -> impl Iterator<Item = &InstalledDist> {
         std::iter::empty()
     }
-
-    fn get_editables(&self, _url: &Url) -> Vec<&InstalledDist> {
-        Vec::new()
-    }
 }
