@@ -7,22 +7,15 @@ use std::path::Path;
 use std::str::FromStr;
 
 use anstream::{eprint, AutoStream, StripStream};
-use anyhow::{anyhow, Context, Result};
+use anyhow::{anyhow, Result};
 use fs_err as fs;
-use indexmap::IndexMap;
 use itertools::Itertools;
 use owo_colors::OwoColorize;
-use tempfile::tempdir_in;
 use tracing::debug;
 
-use distribution_types::{
-    IndexLocations, LocalEditable, LocalEditables, ParsedUrlError, SourceAnnotation,
-    SourceAnnotations, Verbatim,
-};
-use distribution_types::{Requirement, Requirements};
+use distribution_types::{IndexLocations, SourceAnnotation, SourceAnnotations, Verbatim};
 use install_wheel_rs::linker::LinkMode;
 use platform_tags::Tags;
-use requirements_txt::EditableRequirement;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity, FlatIndexClient, RegistryClientBuilder};
@@ -34,7 +27,6 @@ use uv_configuration::{KeyringProviderType, TargetTriple};
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
-use uv_installer::Downloader;
 use uv_interpreter::{
     find_best_interpreter, find_interpreter, InterpreterRequest, PythonEnvironment, SystemPython,
     VersionRequest,
@@ -46,15 +38,15 @@ use uv_requirements::{
     RequirementsSource, RequirementsSpecification, SourceTreeResolver,
 };
 use uv_resolver::{
-    AnnotationStyle, BuiltEditableMetadata, DependencyMode, DisplayResolutionGraph, ExcludeNewer,
-    Exclusions, FlatIndex, InMemoryIndex, Manifest, OptionsBuilder, PreReleaseMode,
-    PythonRequirement, ResolutionMode, Resolver,
+    AnnotationStyle, DependencyMode, DisplayResolutionGraph, ExcludeNewer, Exclusions, FlatIndex,
+    InMemoryIndex, Manifest, OptionsBuilder, PreReleaseMode, PythonRequirement, ResolutionMode,
+    Resolver,
 };
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 use uv_warnings::warn_user;
 
 use crate::commands::pip::operations;
-use crate::commands::reporters::{DownloadReporter, ResolverReporter};
+use crate::commands::reporters::ResolverReporter;
 use crate::commands::{elapsed, ExitStatus};
 use crate::printer::Printer;
 
@@ -124,7 +116,6 @@ pub(crate) async fn pip_compile(
         requirements,
         constraints,
         overrides,
-        editables,
         source_trees,
         extras: used_extras,
         index_url,
@@ -137,6 +128,7 @@ pub(crate) async fn pip_compile(
         requirements,
         constraints,
         overrides,
+        None,
         &extras,
         &client_builder,
         preview,
@@ -171,16 +163,16 @@ pub(crate) async fn pip_compile(
     };
     let interpreter = if let Some(python) = python.as_ref() {
         let request = InterpreterRequest::parse(python);
-        let sources = SourceSelector::from_settings(system);
+        let sources = SourceSelector::from_settings(system, preview);
         find_interpreter(&request, system, &sources, &cache)??
     } else {
         let request = if let Some(version) = python_version.as_ref() {
             // TODO(zanieb): We should consolidate `VersionRequest` and `PythonVersion`
             InterpreterRequest::Version(VersionRequest::from(version))
         } else {
-            InterpreterRequest::Version(VersionRequest::Default)
+            InterpreterRequest::default()
         };
-        find_best_interpreter(&request, system, &cache)??
+        find_best_interpreter(&request, system, preview, &cache)??
     }
     .into_interpreter();
 
@@ -420,98 +412,10 @@ pub(crate) async fn pip_compile(
         }
     }
 
-    for editable in &editables {
-        if let Some(origin) = &editable.origin {
-            sources.add_editable(
-                editable.url(),
-                SourceAnnotation::Requirement(origin.clone()),
-            );
-        }
-    }
-
     // Collect constraints and overrides.
     let constraints = Constraints::from_requirements(constraints);
     let overrides = Overrides::from_requirements(overrides);
 
-    // Build the editables and add their requirements
-    let editables = if editables.is_empty() {
-        Vec::new()
-    } else {
-        let start = std::time::Instant::now();
-
-        let editables = LocalEditables::from_editables(editables.into_iter().map(|editable| {
-            let EditableRequirement {
-                url,
-                extras,
-                path,
-                marker: _,
-                origin: _,
-            } = editable;
-            LocalEditable { url, path, extras }
-        }));
-
-        let downloader = Downloader::new(
-            &cache,
-            &tags,
-            &hasher,
-            DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
-        )
-        .with_reporter(DownloadReporter::from(printer).with_length(editables.len() as u64));
-
-        // Build all editables.
-        let editable_wheel_dir = tempdir_in(cache.root())?;
-        let editables: Vec<BuiltEditableMetadata> = downloader
-            .build_editables(editables, editable_wheel_dir.path())
-            .await
-            .context("Failed to build editables")?
-            .into_iter()
-            .map(|built_editable| {
-                let requirements = Requirements {
-                    dependencies: built_editable
-                        .metadata
-                        .requires_dist
-                        .iter()
-                        .cloned()
-                        .map(Requirement::from_pep508)
-                        .collect::<Result<_, _>>()?,
-                    optional_dependencies: IndexMap::default(),
-                };
-                Ok::<_, Box<ParsedUrlError>>(BuiltEditableMetadata {
-                    built: built_editable.editable,
-                    metadata: built_editable.metadata,
-                    requirements,
-                })
-            })
-            .collect::<Result<_, _>>()?;
-
-        // Validate that the editables are compatible with the target Python version.
-        for editable in &editables {
-            if let Some(python_requires) = editable.metadata.requires_python.as_ref() {
-                if !python_requires.contains(python_requirement.target()) {
-                    return Err(anyhow!(
-                        "Editable `{}` requires Python {}, but resolution targets Python {}",
-                        editable.metadata.name,
-                        python_requires,
-                        python_requirement.target()
-                    ));
-                }
-            }
-        }
-
-        let s = if editables.len() == 1 { "" } else { "s" };
-        writeln!(
-            printer.stderr(),
-            "{}",
-            format!(
-                "Built {} in {}",
-                format!("{} editable{}", editables.len(), s).bold(),
-                elapsed(start.elapsed())
-            )
-            .dimmed()
-        )?;
-        editables
-    };
-
     // Determine any lookahead requirements.
     let lookaheads = match dependency_mode {
         DependencyMode::Transitive => {
@@ -519,7 +423,6 @@ pub(crate) async fn pip_compile(
                 &requirements,
                 &constraints,
                 &overrides,
-                &editables,
                 &hasher,
                 &top_level_index,
                 DistributionDatabase::new(&client, &build_dispatch, concurrency.downloads),
@@ -538,7 +441,6 @@ pub(crate) async fn pip_compile(
         overrides,
         preferences,
         project,
-        editables,
         // Do not consider any installed packages during resolution.
         Exclusions::All,
         lookaheads,
@@ -708,7 +610,7 @@ fn cmd(
     }
     let args = env::args_os()
         .skip(1)
-        .map(|arg| arg.user_display().to_string())
+        .map(|arg| arg.to_string_lossy().to_string())
         .scan(None, move |skip_next, arg| {
             if matches!(skip_next, Some(true)) {
                 // Reset state; skip this iteration.
