@@ -9,15 +9,14 @@ use owo_colors::OwoColorize;
 use tracing::debug;
 
 use distribution_types::{
-    CachedDist, Diagnostic, InstalledDist, Requirement, UnresolvedRequirementSpecification,
+    CachedDist, Diagnostic, InstalledDist, Requirement, ResolutionDiagnostic,
+    UnresolvedRequirementSpecification,
 };
 use distribution_types::{
-    DistributionMetadata, IndexLocations, InstalledMetadata, InstalledVersion, LocalDist, Name,
-    ParsedUrl, RequirementSource, Resolution,
+    DistributionMetadata, IndexLocations, InstalledMetadata, LocalDist, Name, Resolution,
 };
 use install_wheel_rs::linker::LinkMode;
-use pep440_rs::{VersionSpecifier, VersionSpecifiers};
-use pep508_rs::{MarkerEnvironment, VerbatimUrl};
+use pep508_rs::MarkerEnvironment;
 use platform_tags::Tags;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, RegistryClient};
@@ -27,12 +26,12 @@ use uv_configuration::{
 use uv_dispatch::BuildDispatch;
 use uv_distribution::DistributionDatabase;
 use uv_fs::Simplified;
-use uv_installer::{Downloader, Plan, Planner, ResolvedEditable, SitePackages};
+use uv_installer::{Downloader, Plan, Planner, SitePackages};
 use uv_interpreter::{Interpreter, PythonEnvironment};
 use uv_normalize::PackageName;
 use uv_requirements::{
     ExtrasSpecification, LookaheadResolver, NamedRequirementsResolver, RequirementsSource,
-    RequirementsSpecification, SourceTreeResolver,
+    RequirementsSpecification, SourceTreeResolver, Workspace,
 };
 use uv_resolver::{
     DependencyMode, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options, Preference,
@@ -44,7 +43,6 @@ use uv_warnings::warn_user;
 use crate::commands::reporters::{DownloadReporter, InstallReporter, ResolverReporter};
 use crate::commands::DryRunEvent;
 use crate::commands::{compile_bytecode, elapsed, ChangeEvent, ChangeEventKind};
-use crate::editables::ResolvedEditables;
 use crate::printer::Printer;
 
 /// Consolidate the requirements for an installation.
@@ -52,6 +50,7 @@ pub(crate) async fn read_requirements(
     requirements: &[RequirementsSource],
     constraints: &[RequirementsSource],
     overrides: &[RequirementsSource],
+    workspace: Option<&Workspace>,
     extras: &ExtrasSpecification,
     client_builder: &BaseClientBuilder<'_>,
     preview: PreviewMode,
@@ -70,6 +69,7 @@ pub(crate) async fn read_requirements(
         requirements,
         constraints,
         overrides,
+        workspace,
         extras,
         client_builder,
         preview,
@@ -109,7 +109,6 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     source_trees: Vec<PathBuf>,
     project: Option<PackageName>,
     extras: &ExtrasSpecification,
-    editables: &ResolvedEditables,
     installed_packages: InstalledPackages,
     hasher: &HashStrategy,
     reinstall: &Reinstall,
@@ -175,9 +174,6 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     let overrides = Overrides::from_requirements(overrides);
     let python_requirement = PythonRequirement::from_marker_environment(interpreter, markers);
 
-    // Map the editables to their metadata.
-    let editables = editables.as_metadata().map_err(Error::ParsedUrl)?;
-
     // Determine any lookahead requirements.
     let lookaheads = match options.dependency_mode {
         DependencyMode::Transitive => {
@@ -185,7 +181,6 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
                 &requirements,
                 &constraints,
                 &overrides,
-                &editables,
                 hasher,
                 index,
                 DistributionDatabase::new(client, build_dispatch, concurrency.downloads),
@@ -200,38 +195,12 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
     // TODO(zanieb): Consider consuming these instead of cloning
     let exclusions = Exclusions::new(reinstall.clone(), upgrade.clone());
 
-    // Prefer current site packages; filter out packages that are marked for reinstall or upgrade
+    // Prefer current site packages; filter out packages that are marked for reinstall or upgrade.
     let preferences = installed_packages
         .iter()
         .filter(|dist| !exclusions.contains(dist.name()))
-        .map(|dist| {
-            let source = match dist.installed_version() {
-                InstalledVersion::Version(version) => RequirementSource::Registry {
-                    specifier: VersionSpecifiers::from(VersionSpecifier::equals_version(
-                        version.clone(),
-                    )),
-                    // TODO(konstin): track index
-                    index: None,
-                },
-                InstalledVersion::Url(url, _version) => {
-                    let parsed_url = ParsedUrl::try_from(url.clone())?;
-                    RequirementSource::from_parsed_url(
-                        parsed_url,
-                        VerbatimUrl::from_url(url.clone()),
-                    )
-                }
-            };
-            let requirement = Requirement {
-                name: dist.name().clone(),
-                extras: vec![],
-                marker: None,
-                source,
-                origin: None,
-            };
-            Ok(Preference::from_requirement(requirement))
-        })
-        .collect::<Result<_, _>>()
-        .map_err(Error::UnsupportedInstalledDist)?;
+        .map(Preference::from_installed)
+        .collect::<Vec<_>>();
 
     // Create a manifest of the requirements.
     let manifest = Manifest::new(
@@ -240,7 +209,6 @@ pub(crate) async fn resolve<InstalledPackages: InstalledPackagesProvider>(
         overrides,
         preferences,
         project,
-        editables,
         exclusions,
         lookaheads,
     );
@@ -307,7 +275,6 @@ pub(crate) enum Modifications {
 #[allow(clippy::too_many_arguments)]
 pub(crate) async fn install(
     resolution: &Resolution,
-    editables: &[ResolvedEditable],
     site_packages: SitePackages,
     modifications: Modifications,
     reinstall: &Reinstall,
@@ -328,26 +295,12 @@ pub(crate) async fn install(
 ) -> Result<(), Error> {
     let start = std::time::Instant::now();
 
-    // Extract the requirements from the resolution, filtering out any editables that were already
-    // required. If a package is already installed as editable, it may appear in the resolution
-    // despite not being explicitly requested.
-    let requirements = resolution
-        .requirements()
-        .filter(|requirement| {
-            if requirement.source.is_editable() {
-                !editables
-                    .iter()
-                    .any(|editable| requirement.name == *editable.name())
-            } else {
-                true
-            }
-        })
-        .collect::<Vec<_>>();
+    // Extract the requirements from the resolution.
+    let requirements = resolution.requirements().collect::<Vec<_>>();
 
     // Partition into those that should be linked from the cache (`local`), those that need to be
     // downloaded (`remote`), and those that should be removed (`extraneous`).
-    let plan = Planner::with_requirements(&requirements)
-        .with_editable_requirements(editables)
+    let plan = Planner::new(&requirements)
         .build(
             site_packages,
             reinstall,
@@ -708,7 +661,7 @@ pub(crate) fn report_modifications(
 
 /// Report any diagnostics on resolved distributions.
 pub(crate) fn diagnose_resolution(
-    diagnostics: &[Diagnostic],
+    diagnostics: &[ResolutionDiagnostic],
     printer: Printer,
 ) -> Result<(), Error> {
     for diagnostic in diagnostics {
@@ -768,12 +721,6 @@ pub(crate) enum Error {
     #[error(transparent)]
     Lookahead(#[from] uv_requirements::LookaheadError),
 
-    #[error(transparent)]
-    ParsedUrl(Box<distribution_types::ParsedUrlError>),
-
     #[error(transparent)]
     Anyhow(#[from] anyhow::Error),
-
-    #[error("Installed distribution has unsupported type")]
-    UnsupportedInstalledDist(#[source] Box<distribution_types::ParsedUrlError>),
 }
