@@ -7,6 +7,7 @@ use tracing::debug;
 
 use distribution_types::{InstalledMetadata, Name, Requirement, UnresolvedRequirement};
 use pep508_rs::UnnamedRequirement;
+use pypi_types::VerbatimParsedUrl;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, Connectivity};
 use uv_configuration::{KeyringProviderType, PreviewMode};
@@ -48,7 +49,7 @@ pub(crate) async fn pip_uninstall(
     } else {
         SystemPython::Explicit
     };
-    let venv = PythonEnvironment::find(python.as_deref(), system, &cache)?;
+    let venv = PythonEnvironment::find(python.as_deref(), system, preview, &cache)?;
 
     debug!(
         "Using Python {} environment at {}",
@@ -94,7 +95,7 @@ pub(crate) async fn pip_uninstall(
     let site_packages = uv_installer::SitePackages::from_executable(&venv)?;
 
     // Partition the requirements into named and unnamed requirements.
-    let (named, unnamed): (Vec<Requirement>, Vec<UnnamedRequirement>) = spec
+    let (named, unnamed): (Vec<Requirement>, Vec<UnnamedRequirement<VerbatimParsedUrl>>) = spec
         .requirements
         .into_iter()
         .partition_map(|entry| match entry.requirement {
@@ -118,7 +119,7 @@ pub(crate) async fn pip_uninstall(
     let urls = {
         let mut urls = unnamed
             .into_iter()
-            .map(|requirement| requirement.url.to_url())
+            .map(|requirement| requirement.url.verbatim.to_url())
             .collect::<Vec<_>>();
         urls.sort_unstable();
         urls.dedup();
