@@ -7,17 +7,15 @@ use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, RegistryClientBuilder};
 use uv_configuration::{
     Concurrency, ConfigSettings, NoBinary, NoBuild, PreviewMode, Reinstall, SetupPyStrategy,
+    Upgrade,
 };
 use uv_dispatch::BuildDispatch;
-use uv_requirements::{ExtrasSpecification, RequirementsSpecification};
-use uv_resolver::{FlatIndex, InMemoryIndex, OptionsBuilder};
+use uv_requirements::{ExtrasSpecification, ProjectWorkspace, RequirementsSpecification};
+use uv_resolver::{FlatIndex, InMemoryIndex, Options};
 use uv_types::{BuildIsolation, EmptyInstalledPackages, HashStrategy, InFlight};
 use uv_warnings::warn_user;
 
-use crate::commands::project::discovery::Project;
-use crate::commands::project::Error;
-use crate::commands::{project, ExitStatus};
-use crate::editables::ResolvedEditables;
+use crate::commands::{pip, project, ExitStatus};
 use crate::printer::Printer;
 
 /// Resolve the project requirements into a lockfile.
@@ -32,14 +30,10 @@ pub(crate) async fn lock(
     }
 
     // Find the project requirements.
-    let Some(project) = Project::find(std::env::current_dir()?)? else {
-        return Err(anyhow::anyhow!(
-            "Unable to find `pyproject.toml` for project."
-        ));
-    };
+    let project = ProjectWorkspace::discover(std::env::current_dir()?).await?;
 
     // Discover or create the virtual environment.
-    let venv = project::init(&project, cache, printer)?;
+    let venv = project::init_environment(&project, preview, cache, printer)?;
 
     // TODO(zanieb): Support client configuration
     let client_builder = BaseClientBuilder::default();
@@ -48,9 +42,13 @@ pub(crate) async fn lock(
     // TODO(zanieb): Consider allowing constraints and extras
     // TODO(zanieb): Allow specifying extras somehow
     let spec = RequirementsSpecification::from_sources(
+        // TODO(konsti): With workspace (just like with extras), these are the requirements for
+        // syncing. For locking, we want to use the entire workspace with all extras.
+        // See https://github.com/astral-sh/uv/issues/3700
         &project.requirements(),
         &[],
         &[],
+        None,
         &ExtrasSpecification::None,
         &client_builder,
         preview,
@@ -71,7 +69,9 @@ pub(crate) async fn lock(
 
     // TODO(charlie): Respect project configuration.
     let build_isolation = BuildIsolation::default();
+    let concurrency = Concurrency::default();
     let config_settings = ConfigSettings::default();
+    let extras = ExtrasSpecification::default();
     let flat_index = FlatIndex::default();
     let hasher = HashStrategy::default();
     let in_flight = InFlight::default();
@@ -80,9 +80,10 @@ pub(crate) async fn lock(
     let link_mode = LinkMode::default();
     let no_binary = NoBinary::default();
     let no_build = NoBuild::default();
+    let options = Options::default();
+    let reinstall = Reinstall::default();
     let setup_py = SetupPyStrategy::default();
-    let concurrency = Concurrency::default();
-    let reinstall = Reinstall::None;
+    let upgrade = Upgrade::default();
 
     // Create a build dispatch.
     let build_dispatch = BuildDispatch::new(
@@ -102,37 +103,18 @@ pub(crate) async fn lock(
         concurrency,
     );
 
-    let options = OptionsBuilder::new()
-        // TODO(zanieb): Support resolver options
-        // .resolution_mode(resolution_mode)
-        // .prerelease_mode(prerelease_mode)
-        // .dependency_mode(dependency_mode)
-        // .exclude_newer(exclude_newer)
-        .build();
-
-    // Build all editable distributions. The editables are shared between resolution and
-    // installation, and should live for the duration of the command.
-    let editables = ResolvedEditables::resolve(
-        spec.editables.clone(),
-        &EmptyInstalledPackages,
-        &reinstall,
-        &hasher,
-        &interpreter,
-        tags,
-        cache,
-        &client,
-        &build_dispatch,
-        concurrency,
-        printer,
-    )
-    .await?;
-
     // Resolve the requirements.
-    let resolution = project::resolve(
-        spec,
+    let resolution = pip::operations::resolve(
+        spec.requirements,
+        spec.constraints,
+        spec.overrides,
+        spec.source_trees,
+        spec.project,
+        &extras,
         EmptyInstalledPackages,
-        &editables,
         &hasher,
+        &reinstall,
+        &upgrade,
         &interpreter,
         tags,
         markers,
@@ -140,14 +122,14 @@ pub(crate) async fn lock(
         &flat_index,
         &index,
         &build_dispatch,
+        concurrency,
         options,
         printer,
-        concurrency,
     )
     .await;
 
     let resolution = match resolution {
-        Err(Error::Resolve(uv_resolver::ResolveError::NoSolution(err))) => {
+        Err(pip::operations::Error::Resolve(uv_resolver::ResolveError::NoSolution(err))) => {
             let report = miette::Report::msg(format!("{err}"))
                 .context("No solution found when resolving dependencies:");
             eprint!("{report:?}");
@@ -159,7 +141,11 @@ pub(crate) async fn lock(
     // Write the lockfile to disk.
     let lock = resolution.lock()?;
     let encoded = toml::to_string_pretty(&lock)?;
-    fs_err::tokio::write(project.root().join("uv.lock"), encoded.as_bytes()).await?;
+    fs_err::tokio::write(
+        project.workspace().root().join("uv.lock"),
+        encoded.as_bytes(),
+    )
+    .await?;
 
     Ok(ExitStatus::Success)
 }
