@@ -19,12 +19,11 @@ use uv_fs::Simplified;
 use uv_installer::{SatisfiesResult, SitePackages};
 use uv_interpreter::{find_default_interpreter, PythonEnvironment};
 use uv_requirements::{
-    ExtrasSpecification, ProjectWorkspace, RequirementsSource, RequirementsSpecification,
+    ExtrasSpecification, ProjectWorkspace, RequirementsSource, RequirementsSpecification, Workspace,
 };
 use uv_resolver::{FlatIndex, InMemoryIndex, Options};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
 
-use crate::editables::ResolvedEditables;
 use crate::printer::Printer;
 
 pub(crate) mod lock;
@@ -46,6 +45,7 @@ pub(crate) enum Error {
 /// Initialize a virtual environment for the current project.
 pub(crate) fn init_environment(
     project: &ProjectWorkspace,
+    preview: PreviewMode,
     cache: &Cache,
     printer: Printer,
 ) -> Result<PythonEnvironment, Error> {
@@ -57,7 +57,7 @@ pub(crate) fn init_environment(
         Ok(venv) => Ok(venv),
         Err(uv_interpreter::Error::NotFound(_)) => {
             // TODO(charlie): Respect `--python`; if unset, respect `Requires-Python`.
-            let interpreter = find_default_interpreter(cache)
+            let interpreter = find_default_interpreter(preview, cache)
                 .map_err(uv_interpreter::Error::from)?
                 .map_err(uv_interpreter::Error::from)?
                 .into_interpreter();
@@ -91,6 +91,7 @@ pub(crate) fn init_environment(
 pub(crate) async fn update_environment(
     venv: PythonEnvironment,
     requirements: &[RequirementsSource],
+    workspace: Option<&Workspace>,
     preview: PreviewMode,
     connectivity: Connectivity,
     cache: &Cache,
@@ -106,6 +107,7 @@ pub(crate) async fn update_environment(
         requirements,
         &[],
         &[],
+        workspace,
         &ExtrasSpecification::None,
         &client_builder,
         preview,
@@ -114,10 +116,9 @@ pub(crate) async fn update_environment(
 
     // Check if the current environment satisfies the requirements
     let site_packages = SitePackages::from_executable(&venv)?;
-
-    // If the requirements are already satisfied, we're done.
     if spec.source_trees.is_empty() {
-        match site_packages.satisfies(&spec.requirements, &spec.editables, &spec.constraints)? {
+        match site_packages.satisfies(&spec.requirements, &spec.constraints)? {
+            // If the requirements are already satisfied, we're done.
             SatisfiesResult::Fresh {
                 recursive_requirements,
             } => {
@@ -129,10 +130,6 @@ pub(crate) async fn update_environment(
                         .sorted()
                         .join(" | ")
                 );
-                debug!(
-                    "All editables satisfied: {}",
-                    spec.editables.iter().map(ToString::to_string).join(", ")
-                );
                 return Ok(venv);
             }
             SatisfiesResult::Unsatisfied(requirement) => {
@@ -192,26 +189,6 @@ pub(crate) async fn update_environment(
         concurrency,
     );
 
-    // Build all editable distributions. The editables are shared between resolution and
-    // installation, and should live for the duration of the command.
-    let editables = ResolvedEditables::resolve(
-        spec.editables
-            .iter()
-            .cloned()
-            .map(ResolvedEditables::from_requirement),
-        &site_packages,
-        &reinstall,
-        &hasher,
-        &interpreter,
-        tags,
-        cache,
-        &client,
-        &resolve_dispatch,
-        concurrency,
-        printer,
-    )
-    .await?;
-
     // Resolve the requirements.
     let resolution = match pip::operations::resolve(
         spec.requirements,
@@ -220,7 +197,6 @@ pub(crate) async fn update_environment(
         spec.source_trees,
         spec.project,
         &extras,
-        &editables,
         site_packages.clone(),
         &hasher,
         &reinstall,
@@ -271,7 +247,6 @@ pub(crate) async fn update_environment(
     // Sync the environment.
     pip::operations::install(
         &resolution,
-        &editables,
         site_packages,
         pip::operations::Modifications::Sufficient,
         &reinstall,
