@@ -1,19 +1,19 @@
 use std::ffi::OsString;
 use std::path::PathBuf;
 
-use anyhow::Result;
+use anyhow::{Context, Result};
 use itertools::Itertools;
 use tempfile::tempdir_in;
 use tokio::process::Command;
 use tracing::debug;
 
 use uv_cache::Cache;
+use uv_client::Connectivity;
 use uv_configuration::PreviewMode;
-use uv_interpreter::PythonEnvironment;
-use uv_requirements::RequirementsSource;
+use uv_interpreter::{PythonEnvironment, SystemPython};
+use uv_requirements::{ProjectWorkspace, RequirementsSource};
 use uv_warnings::warn_user;
 
-use crate::commands::project::discovery::Project;
 use crate::commands::{project, ExitStatus};
 use crate::printer::Printer;
 
@@ -26,6 +26,7 @@ pub(crate) async fn run(
     python: Option<String>,
     isolated: bool,
     preview: PreviewMode,
+    connectivity: Connectivity,
     cache: &Cache,
     printer: Printer,
 ) -> Result<ExitStatus> {
@@ -55,18 +56,22 @@ pub(crate) async fn run(
     } else {
         debug!("Syncing project environment.");
 
-        let Some(project) = Project::find(std::env::current_dir()?)? else {
-            return Err(anyhow::anyhow!(
-                "Unable to find `pyproject.toml` for project."
-            ));
-        };
+        let project = ProjectWorkspace::discover(std::env::current_dir()?).await?;
 
-        let venv = project::init(&project, cache, printer)?;
+        let venv = project::init_environment(&project, preview, cache, printer)?;
 
         // Install the project requirements.
         Some(
-            project::update_environment(venv, &project.requirements(), preview, cache, printer)
-                .await?,
+            project::update_environment(
+                venv,
+                &project.requirements(),
+                Some(project.workspace()),
+                preview,
+                connectivity,
+                cache,
+                printer,
+            )
+            .await?,
         )
     };
 
@@ -81,9 +86,10 @@ pub(crate) async fn run(
         let interpreter = if let Some(project_env) = &project_env {
             project_env.interpreter().clone()
         } else if let Some(python) = python.as_ref() {
-            PythonEnvironment::from_requested_python(python, cache)?.into_interpreter()
+            PythonEnvironment::from_requested_python(python, SystemPython::Allowed, preview, cache)?
+                .into_interpreter()
         } else {
-            PythonEnvironment::from_default_python(cache)?.into_interpreter()
+            PythonEnvironment::from_default_python(preview, cache)?.into_interpreter()
         };
 
         // TODO(charlie): If the environment satisfies the requirements, skip creation.
@@ -105,7 +111,18 @@ pub(crate) async fn run(
         )?;
 
         // Install the ephemeral requirements.
-        Some(project::update_environment(venv, &requirements, preview, cache, printer).await?)
+        Some(
+            project::update_environment(
+                venv,
+                &requirements,
+                None,
+                preview,
+                connectivity,
+                cache,
+                printer,
+            )
+            .await?,
+        )
     };
 
     // Construct the command
@@ -166,8 +183,10 @@ pub(crate) async fn run(
         "Running `{command}{space}{}`",
         args.iter().map(|arg| arg.to_string_lossy()).join(" ")
     );
-    let mut handle = process.spawn()?;
-    let status = handle.wait().await?;
+    let mut handle = process
+        .spawn()
+        .with_context(|| format!("Failed to spawn: `{command}`"))?;
+    let status = handle.wait().await.context("Child process disappeared")?;
 
     // Exit based on the result of the command
     // TODO(zanieb): Do we want to exit with the code of the child process? Probably.
