@@ -1,7 +1,7 @@
 use std::ffi::OsString;
 use std::path::PathBuf;
 
-use anyhow::Result;
+use anyhow::{Context, Result};
 use itertools::Itertools;
 use tempfile::tempdir_in;
 use tokio::process::Command;
@@ -25,6 +25,7 @@ pub(crate) async fn run(
     args: Vec<OsString>,
     python: Option<String>,
     from: Option<String>,
+    with: Vec<String>,
     _isolated: bool,
     preview: PreviewMode,
     connectivity: Connectivity,
@@ -35,10 +36,12 @@ pub(crate) async fn run(
         warn_user!("`uv tool run` is experimental and may change without warning.");
     }
 
-    // TODO(zanieb): Allow users to pass additional requirements
     let requirements = [RequirementsSource::from_package(
         from.unwrap_or_else(|| target.clone()),
-    )];
+    )]
+    .into_iter()
+    .chain(with.into_iter().map(RequirementsSource::from_package))
+    .collect::<Vec<_>>();
 
     // TODO(zanieb): When implementing project-level tools, discover the project and check if it has the tool
     // TOOD(zanieb): Determine if we sould layer on top of the project environment if it is present
@@ -48,10 +51,10 @@ pub(crate) async fn run(
 
     // Discover an interpreter.
     let interpreter = if let Some(python) = python.as_ref() {
-        PythonEnvironment::from_requested_python(python, SystemPython::Allowed, cache)?
+        PythonEnvironment::from_requested_python(python, SystemPython::Allowed, preview, cache)?
             .into_interpreter()
     } else {
-        PythonEnvironment::from_default_python(cache)?.into_interpreter()
+        PythonEnvironment::from_default_python(preview, cache)?.into_interpreter()
     };
 
     // Create a virtual environment
@@ -68,8 +71,18 @@ pub(crate) async fn run(
     )?;
 
     // Install the ephemeral requirements.
-    let ephemeral_env =
-        Some(update_environment(venv, &requirements, preview, connectivity, cache, printer).await?);
+    let ephemeral_env = Some(
+        update_environment(
+            venv,
+            &requirements,
+            None,
+            preview,
+            connectivity,
+            cache,
+            printer,
+        )
+        .await?,
+    );
 
     // TODO(zanieb): Determine the command via the package entry points
     let command = target;
@@ -119,8 +132,10 @@ pub(crate) async fn run(
         "Running `{command}{space}{}`",
         args.iter().map(|arg| arg.to_string_lossy()).join(" ")
     );
-    let mut handle = process.spawn()?;
-    let status = handle.wait().await?;
+    let mut handle = process
+        .spawn()
+        .with_context(|| format!("Failed to spawn: `{command}`"))?;
+    let status = handle.wait().await.context("Child process disappeared")?;
 
     // Exit based on the result of the command
     // TODO(zanieb): Do we want to exit with the code of the child process? Probably.
