@@ -10,6 +10,7 @@ use clap::{CommandFactory, Parser};
 use owo_colors::OwoColorize;
 use tracing::instrument;
 
+use cli::{ToolCommand, ToolNamespace};
 use uv_cache::Cache;
 use uv_requirements::RequirementsSource;
 use uv_workspace::Combine;
@@ -43,7 +44,6 @@ static GLOBAL: tikv_jemallocator::Jemalloc = tikv_jemallocator::Jemalloc;
 mod cli;
 mod commands;
 mod compat;
-mod editables;
 mod logging;
 mod printer;
 mod settings;
@@ -184,6 +184,7 @@ async fn run() -> Result<ExitStatus> {
 
             // Initialize the cache.
             let cache = cache.init()?.with_refresh(args.refresh);
+
             let requirements = args
                 .src_file
                 .into_iter()
@@ -225,7 +226,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.keyring_provider,
                 args.shared.setup_py,
                 args.shared.config_setting,
-                args.shared.connectivity,
+                globals.connectivity,
                 args.shared.no_build_isolation,
                 args.shared.no_build,
                 args.shared.python_version,
@@ -259,14 +260,21 @@ async fn run() -> Result<ExitStatus> {
 
             // Initialize the cache.
             let cache = cache.init()?.with_refresh(args.refresh);
-            let sources = args
+
+            let requirements = args
                 .src_file
                 .into_iter()
                 .map(RequirementsSource::from_requirements_file)
                 .collect::<Vec<_>>();
+            let constraints = args
+                .constraint
+                .into_iter()
+                .map(RequirementsSource::from_constraints_txt)
+                .collect::<Vec<_>>();
 
             commands::pip_sync(
-                &sources,
+                &requirements,
+                &constraints,
                 &args.reinstall,
                 args.shared.link_mode,
                 args.shared.compile_bytecode,
@@ -275,7 +283,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.index_strategy,
                 args.shared.keyring_provider,
                 args.shared.setup_py,
-                args.shared.connectivity,
+                globals.connectivity,
                 &args.shared.config_setting,
                 args.shared.no_build_isolation,
                 args.shared.no_build,
@@ -283,6 +291,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.python_version,
                 args.shared.python_platform,
                 args.shared.strict,
+                args.shared.exclude_newer,
                 args.shared.python,
                 args.shared.system,
                 args.shared.break_system_packages,
@@ -291,6 +300,7 @@ async fn run() -> Result<ExitStatus> {
                 globals.native_tls,
                 globals.preview,
                 cache,
+                args.dry_run,
                 printer,
             )
             .await
@@ -348,7 +358,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.compile_bytecode,
                 args.shared.require_hashes,
                 args.shared.setup_py,
-                args.shared.connectivity,
+                globals.connectivity,
                 &args.shared.config_setting,
                 args.shared.no_build_isolation,
                 args.shared.no_build,
@@ -397,7 +407,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.break_system_packages,
                 args.shared.target,
                 cache,
-                args.shared.connectivity,
+                globals.connectivity,
                 globals.native_tls,
                 globals.preview,
                 args.shared.keyring_provider,
@@ -419,6 +429,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.strict,
                 args.shared.python.as_deref(),
                 args.shared.system,
+                globals.preview,
                 &cache,
                 printer,
             )
@@ -442,6 +453,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.strict,
                 args.shared.python.as_deref(),
                 args.shared.system,
+                globals.preview,
                 &cache,
                 printer,
             )
@@ -460,6 +472,7 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.strict,
                 args.shared.python.as_deref(),
                 args.shared.system,
+                globals.preview,
                 &cache,
                 printer,
             )
@@ -476,6 +489,7 @@ async fn run() -> Result<ExitStatus> {
             commands::pip_check(
                 args.shared.python.as_deref(),
                 args.shared.system,
+                globals.preview,
                 &cache,
                 printer,
             )
@@ -520,11 +534,12 @@ async fn run() -> Result<ExitStatus> {
                 args.shared.keyring_provider,
                 uv_virtualenv::Prompt::from_args(prompt),
                 args.system_site_packages,
-                args.shared.connectivity,
+                globals.connectivity,
                 args.seed,
                 args.allow_existing,
                 args.shared.exclude_newer,
                 globals.native_tls,
+                globals.preview,
                 &cache,
                 printer,
             )
@@ -564,6 +579,7 @@ async fn run() -> Result<ExitStatus> {
                 args.python,
                 globals.isolated,
                 globals.preview,
+                globals.connectivity,
                 &cache,
                 printer,
             )
@@ -599,6 +615,26 @@ async fn run() -> Result<ExitStatus> {
             shell.generate(&mut Cli::command(), &mut stdout());
             Ok(ExitStatus::Success)
         }
+        Commands::Tool(ToolNamespace {
+            command: ToolCommand::Run(args),
+        }) => {
+            // Initialize the cache.
+            let cache = cache.init()?;
+
+            commands::run_tool(
+                args.target,
+                args.args,
+                args.python,
+                args.from,
+                args.with,
+                globals.isolated,
+                globals.preview,
+                globals.connectivity,
+                &cache,
+                printer,
+            )
+            .await
+        }
     }
 }
 
