@@ -34,6 +34,7 @@ pub(crate) struct GlobalSettings {
     pub(crate) verbose: u8,
     pub(crate) color: ColorChoice,
     pub(crate) native_tls: bool,
+    pub(crate) connectivity: Connectivity,
     pub(crate) isolated: bool,
     pub(crate) preview: PreviewMode,
 }
@@ -52,6 +53,14 @@ impl GlobalSettings {
             native_tls: flag(args.native_tls, args.no_native_tls)
                 .combine(workspace.and_then(|workspace| workspace.options.native_tls))
                 .unwrap_or(false),
+            connectivity: if flag(args.offline, args.no_offline)
+                .combine(workspace.and_then(|workspace| workspace.options.offline))
+                .unwrap_or(false)
+            {
+                Connectivity::Offline
+            } else {
+                Connectivity::Online
+            },
             isolated: args.isolated,
             preview: PreviewMode::from(
                 flag(args.preview, args.no_preview)
@@ -199,8 +208,7 @@ impl PipCompileSettings {
             header,
             annotation_style,
             custom_compile_command,
-            offline,
-            no_offline,
+
             refresh,
             no_refresh,
             refresh_package,
@@ -261,7 +269,7 @@ impl PipCompileSettings {
                 PipOptions {
                     python,
                     system: flag(system, no_system),
-                    offline: flag(offline, no_offline),
+
                     index_url: index_url.and_then(Maybe::into_option),
                     extra_index_url: extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
@@ -322,8 +330,10 @@ impl PipCompileSettings {
 pub(crate) struct PipSyncSettings {
     // CLI-only settings.
     pub(crate) src_file: Vec<PathBuf>,
+    pub(crate) constraint: Vec<PathBuf>,
     pub(crate) reinstall: Reinstall,
     pub(crate) refresh: Refresh,
+    pub(crate) dry_run: bool,
 
     // Shared settings.
     pub(crate) shared: PipSharedSettings,
@@ -334,12 +344,11 @@ impl PipSyncSettings {
     pub(crate) fn resolve(args: PipSyncArgs, workspace: Option<Workspace>) -> Self {
         let PipSyncArgs {
             src_file,
+            constraint,
             reinstall,
             no_reinstall,
             reinstall_package,
-            offline,
             refresh,
-            no_offline,
             no_refresh,
             refresh_package,
             link_mode,
@@ -372,14 +381,21 @@ impl PipSyncSettings {
             python_platform,
             strict,
             no_strict,
+            exclude_newer,
+            dry_run,
             compat_args: _,
         } = args;
 
         Self {
             // CLI-only settings.
             src_file,
+            constraint: constraint
+                .into_iter()
+                .filter_map(Maybe::into_option)
+                .collect(),
             reinstall: Reinstall::from_args(flag(reinstall, no_reinstall), reinstall_package),
             refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
+            dry_run,
 
             // Shared settings.
             shared: PipSharedSettings::combine(
@@ -388,7 +404,7 @@ impl PipSyncSettings {
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
                     target,
-                    offline: flag(offline, no_offline),
+
                     index_url: index_url.and_then(Maybe::into_option),
                     extra_index_url: extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
@@ -411,6 +427,7 @@ impl PipSyncSettings {
                     }),
                     python_version,
                     python_platform,
+                    exclude_newer,
                     link_mode,
                     compile_bytecode: flag(compile_bytecode, no_compile_bytecode),
                     require_hashes: flag(require_hashes, no_require_hashes),
@@ -440,6 +457,7 @@ pub(crate) struct PipInstallSettings {
     pub(crate) refresh: Refresh,
     pub(crate) dry_run: bool,
     pub(crate) uv_lock: Option<String>,
+
     // Shared settings.
     pub(crate) shared: PipSharedSettings,
 }
@@ -462,9 +480,7 @@ impl PipInstallSettings {
             reinstall,
             no_reinstall,
             reinstall_package,
-            offline,
             refresh,
-            no_offline,
             no_refresh,
             refresh_package,
             no_deps,
@@ -531,7 +547,7 @@ impl PipInstallSettings {
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
                     target,
-                    offline: flag(offline, no_offline),
+
                     index_url: index_url.and_then(Maybe::into_option),
                     extra_index_url: extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
@@ -602,8 +618,6 @@ impl PipUninstallSettings {
             break_system_packages,
             no_break_system_packages,
             target,
-            offline,
-            no_offline,
         } = args;
 
         Self {
@@ -618,7 +632,7 @@ impl PipUninstallSettings {
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
                     target,
-                    offline: flag(offline, no_offline),
+
                     keyring_provider,
                     ..PipOptions::default()
                 },
@@ -826,8 +840,7 @@ impl VenvSettings {
             no_index,
             index_strategy,
             keyring_provider,
-            offline,
-            no_offline,
+
             exclude_newer,
             compat_args: _,
         } = args;
@@ -845,7 +858,7 @@ impl VenvSettings {
                 PipOptions {
                     python,
                     system: flag(system, no_system),
-                    offline: flag(offline, no_offline),
+
                     index_url: index_url.and_then(Maybe::into_option),
                     extra_index_url: extra_index_url.map(|extra_index_urls| {
                         extra_index_urls
@@ -878,7 +891,6 @@ pub(crate) struct PipSharedSettings {
     pub(crate) extras: ExtrasSpecification,
     pub(crate) break_system_packages: bool,
     pub(crate) target: Option<Target>,
-    pub(crate) connectivity: Connectivity,
     pub(crate) index_strategy: IndexStrategy,
     pub(crate) keyring_provider: KeyringProviderType,
     pub(crate) no_binary: NoBinary,
@@ -919,7 +931,6 @@ impl PipSharedSettings {
             system,
             break_system_packages,
             target,
-            offline,
             index_url,
             extra_index_url,
             no_index,
@@ -995,11 +1006,6 @@ impl PipSharedSettings {
                 .annotation_style
                 .combine(annotation_style)
                 .unwrap_or_default(),
-            connectivity: if args.offline.combine(offline).unwrap_or_default() {
-                Connectivity::Offline
-            } else {
-                Connectivity::Online
-            },
             index_strategy: args
                 .index_strategy
                 .combine(index_strategy)
